@@ -26,6 +26,7 @@
 #include <stdlib.h>
 #include <time.h>
 #include <unistd.h>
+#include <arpa/inet.h>
 
 #include <nss.h>
 #include <certt.h>
@@ -40,6 +41,10 @@
 
 #include <krb5.h>
 
+#ifdef CM_USE_IDN
+#include <idna.h>
+#endif
+
 #include "certext.h"
 #include "certext-n.h"
 #include "log.h"
@@ -305,7 +310,7 @@ cm_certext_read_ku(struct cm_store_entry *entry, PLArenaPool *arena,
 		entry->cm_cert_ku = talloc_zero_size(entry, item.len + 1);
 		for (i = 0; i < item.len; i++) {
 			bit = (item.data[i / 8] & (0x80 >> (i % 8))) ? 1 : 0;
-			sprintf(entry->cm_cert_ku + i, "%.*d", 1, bit);
+			sprintf(entry->cm_cert_ku + i, "%.*u", 1, bit);
 		}
 	}
 }
@@ -318,6 +323,7 @@ cm_certext_build_ku(struct cm_store_entry *entry, PLArenaPool *arena,
 {
 	SECItem *ret, encoded, *bits;
 	unsigned int i, used, val, len;
+
 	if ((ku_value == NULL) || (strlen(ku_value) == 0)) {
 		/* Nothing to encode, so don't include this extension. */
 		return NULL;
@@ -520,6 +526,7 @@ cm_certext_build_eku(struct cm_store_entry *entry, PLArenaPool *arena,
 	char *numeric, *symbolic;
 	void *tctx;
 	SECItem **oids = NULL, **tmp, encoded, *ret;
+
 	if ((eku_value == NULL) || (strlen(eku_value) == 0)) {
 		return NULL;
 	}
@@ -871,7 +878,8 @@ cm_certext_read_san(struct cm_store_entry *entry, PLArenaPool *arena,
 {
 	CERTGeneralName *name, *san;
 	unsigned int i, j;
-	char **s;
+	char **s, abuf[64];
+
 	name = CERT_DecodeAltNameExtension(arena, &san_ext->value);
 	san = name;
 	i = 0;
@@ -903,7 +911,29 @@ cm_certext_read_san(struct cm_store_entry *entry, PLArenaPool *arena,
 			cm_certext_remove_duplicates(entry->cm_cert_hostname);
 			break;
 		case certIPAddress:
-			/* binary data - see rfc5280 - XXX */
+			/* An IPv4 or IPv6 address. */
+			if (!((san->name.other.len == 16) &&
+			      (inet_ntop(AF_INET6, san->name.other.data,
+					 abuf, sizeof(abuf)) != NULL)) &&
+			    !((san->name.other.len == 4) &&
+			      (inet_ntop(AF_INET, san->name.other.data,
+					 abuf, sizeof(abuf)) != NULL))) {
+				continue;
+			}
+			for (j = 0;
+			     (entry->cm_cert_ipaddress != NULL) &&
+			     (entry->cm_cert_ipaddress[j] != NULL);
+			     j++) {
+				continue;
+			}
+			s = talloc_zero_array(entry, char *, j + 2);
+			if (j > 0) {
+				memcpy(s, entry->cm_cert_ipaddress,
+				       sizeof(char *) * j);
+			}
+			s[j] = talloc_strdup(entry, abuf);
+			entry->cm_cert_ipaddress = s;
+			cm_certext_remove_duplicates(entry->cm_cert_ipaddress);
 			break;
 		case certRFC822Name:
 			/* An email address is just a string. */
@@ -1031,23 +1061,45 @@ cm_certext_build_principal(struct cm_store_entry *entry, PLArenaPool *arena,
 /* Build up a subjectAltName extension value using information for the entry. */
 static SECItem *
 cm_certext_build_san(struct cm_store_entry *entry, PLArenaPool *arena,
-		     char **hostname, char **email, char **principal)
+		     char **hostname, char **email, char **principal,
+		     char **ipaddress)
 {
 	CERTGeneralName *name, *next;
 	SECItem encoded, *item;
 	int i, j;
+	struct in_addr ip;
+	struct in6_addr ip6;
+	char *p;
+
 	/* Anything to do? */
-	if ((hostname == NULL) && (email == NULL) && (principal == NULL)) {
+	if ((hostname == NULL) && (email == NULL) && (principal == NULL) &&
+	    (ipaddress == NULL)) {
 		return NULL;
 	}
 	name = NULL;
 	/* Build a list of dnsName values. */
 	for (i = 0; (hostname != NULL) && (hostname[i] != NULL); i++) {
+		if (strlen(hostname[i]) == 0) {
+			continue;
+		}
 		next = PORT_ArenaZAlloc(arena, sizeof(*next));
 		if (next != NULL) {
 			next->type = certDNSName;
-			next->name.other.len = strlen(hostname[i]);
-			next->name.other.data = (unsigned char *) hostname[i];
+			p = hostname[i];
+#ifdef CM_USE_IDN
+			if (idna_to_ascii_lz(p, &p, 0) != IDNA_SUCCESS) {
+				cm_log(1, "Unable to convert hostname \"%s\" "
+				       "to an ASCII-compatible name.\n",
+				       hostname[i]);
+				continue;
+			}
+#endif
+			next->name.other.data =
+				(unsigned char *) PORT_ArenaStrdup(arena, p);
+			next->name.other.len = strlen(p);
+			if (p != hostname[i]) {
+				free(p);
+			}
 			if (name == NULL) {
 				name = next;
 				PR_INIT_CLIST(&name->l);
@@ -1058,6 +1110,9 @@ cm_certext_build_san(struct cm_store_entry *entry, PLArenaPool *arena,
 	}
 	/* Build a list of email address values. */
 	for (i = 0; (email != NULL) && (email[i] != NULL); i++) {
+		if (strlen(email[i]) == 0) {
+			continue;
+		}
 		next = PORT_ArenaZAlloc(arena, sizeof(*next));
 		if (next != NULL) {
 			next->type = certRFC822Name;
@@ -1074,6 +1129,9 @@ cm_certext_build_san(struct cm_store_entry *entry, PLArenaPool *arena,
 	/* Build a list of otherName values. Encode every principal name in two
 	 * forms. */
 	for (i = 0; (principal != NULL) && (principal[i] != NULL); i++) {
+		if (strlen(principal[i]) == 0) {
+			continue;
+		}
 		for (j = 0; (j < i) && (principal[j] != NULL); j++) {
 			if (strcmp(principal[i], principal[j]) == 0) {
 				/* We've already seen [i]; skip it. */
@@ -1114,6 +1172,40 @@ cm_certext_build_san(struct cm_store_entry *entry, PLArenaPool *arena,
 			}
 		}
 	}
+	/* Build a list of IP address values. */
+	for (i = 0; (ipaddress != NULL) && (ipaddress[i] != NULL); i++) {
+		if (strlen(ipaddress[i]) == 0) {
+			continue;
+		}
+		next = PORT_ArenaZAlloc(arena, sizeof(*next));
+		if (next != NULL) {
+			next->type = certIPAddress;
+			memset(&encoded, 0, sizeof(encoded));
+			if (inet_pton(AF_INET6, ipaddress[i], &ip6) == 1) {
+				encoded.len = 16;
+				encoded.data = (unsigned char *) &ip6;
+			} else if (inet_pton(AF_INET, ipaddress[i], &ip) == 1) {
+				encoded.len = 4;
+				encoded.data = (unsigned char *) &ip;
+			} else {
+				cm_log(1, "Internal error: unable to parse "
+				       "\"%s\" as an IP address, ignoring.\n",
+				       ipaddress[i]);
+				continue;
+			}
+			item = SECITEM_ArenaDupItem(arena, &encoded);
+			if (item == NULL) {
+				continue;
+			}
+			next->name.other = *item;
+			if (name == NULL) {
+				name = next;
+				PR_INIT_CLIST(&name->l);
+			} else {
+				PR_APPEND_LINK(&next->l, &name->l);
+			}
+		}
+	}
 	/* Encode all of the values. */
 	memset(&encoded, 0, sizeof(encoded));
 	if ((name != NULL) &&
@@ -1156,6 +1248,7 @@ cm_certext_build_self_akid(struct cm_store_entry *entry, PLArenaPool *arena)
 	SECItem pubkeyinfo, pubkey, encoded, *item;
 	unsigned char digest[CM_DIGEST_MAX];
 	const char *pubkey_info;
+	size_t len;
 
 	memset(&pubkey, 0, sizeof(pubkey));
 	if (entry->cm_key_pubkey != NULL) {
@@ -1180,9 +1273,10 @@ cm_certext_build_self_akid(struct cm_store_entry *entry, PLArenaPool *arena)
 							   pubkeyinfo.len);
 			spki = NULL;
 			if (pubkeyinfo.data != NULL) {
-				pubkeyinfo.len = cm_store_hex_to_bin(pubkey_info,
-								     pubkeyinfo.data,
-								     pubkeyinfo.len);
+				len = cm_store_hex_to_bin(pubkey_info,
+							  pubkeyinfo.data,
+							  pubkeyinfo.len);
+				pubkeyinfo.len = len;
 				spki = SECKEY_DecodeDERSubjectPublicKeyInfo(&pubkeyinfo);
 			}
 			if (spki != NULL) {
@@ -1227,14 +1321,16 @@ cm_certext_build_skid(struct cm_store_entry *entry, PLArenaPool *arena)
 	SECItem pubkeyinfo, pubkey, value, encoded, *item;
 	unsigned char digest[CM_DIGEST_MAX];
 	const char *pubkey_info;
+	size_t len;
 
 	memset(&pubkey, 0, sizeof(pubkey));
 	if (entry->cm_key_pubkey != NULL) {
 		pubkey.len = strlen(entry->cm_key_pubkey) / 2;
 		pubkey.data = PORT_ArenaZAlloc(arena, pubkey.len);
 		if (pubkey.data != NULL) {
-			pubkey.len = cm_store_hex_to_bin(entry->cm_key_pubkey,
-							 pubkey.data, pubkey.len);
+			len = cm_store_hex_to_bin(entry->cm_key_pubkey,
+						  pubkey.data, pubkey.len);
+			pubkey.len = len;
 		}
 	}
 	if (pubkey.data == NULL) {
@@ -1250,9 +1346,10 @@ cm_certext_build_skid(struct cm_store_entry *entry, PLArenaPool *arena)
 							   pubkeyinfo.len);
 			spki = NULL;
 			if (pubkeyinfo.data != NULL) {
-				pubkeyinfo.len = cm_store_hex_to_bin(pubkey_info,
-								     pubkeyinfo.data,
-								     pubkeyinfo.len);
+				len = cm_store_hex_to_bin(pubkey_info,
+							  pubkeyinfo.data,
+							  pubkeyinfo.len);
+				pubkeyinfo.len = len;
 				spki = SECKEY_DecodeDERSubjectPublicKeyInfo(&pubkeyinfo);
 			}
 			if (spki != NULL) {
@@ -1299,7 +1396,7 @@ cm_certext_build_aia(struct cm_store_entry *entry, PLArenaPool *arena,
 	SECItem encoded, *item;
 	SECOidData *oid;
 	unsigned char *tmp;
-	unsigned int i, n;
+	unsigned int i, j, n;
 
 	oid = SECOID_FindOIDByTag(SEC_OID_PKIX_OCSP);
 	if (oid == NULL) {
@@ -1325,14 +1422,18 @@ cm_certext_build_aia(struct cm_store_entry *entry, PLArenaPool *arena,
 	if (values == NULL) {
 		return NULL;
 	}
-	for (i = 0; i < n; i++) {
-		location[i].type = certURI;
+	for (i = 0, j = 0; i < n; i++) {
+		if (strlen(ocsp_location[i]) == 0) {
+			continue;
+		}
+		location[j].type = certURI;
 		tmp = (unsigned char *) ocsp_location[i];
-		location[i].name.other.data = tmp;
-		location[i].name.other.len = strlen(ocsp_location[i]);
-		value[i].method = oid->oid;
-		value[i].location = &location[i];
-		values[i] = &value[i];
+		location[j].name.other.data = tmp;
+		location[j].name.other.len = strlen(ocsp_location[i]);
+		value[j].method = oid->oid;
+		value[j].location = &location[j];
+		values[j] = &value[j];
+		j++;
 	}
 	memset(&encoded, 0, sizeof(encoded));
 	if (CERT_EncodeInfoAccessExtension(arena, values,
@@ -1354,7 +1455,7 @@ cm_certext_build_crldp(struct cm_store_entry *entry, PLArenaPool *arena,
 	CERTGeneralName *location;
 	SECItem encoded, *item;
 	SECOidData *oid;
-	unsigned int i, n;
+	unsigned int i, j, n;
 
 	oid = SECOID_FindOIDByTag(SEC_OID_PKIX_OCSP);
 	if (oid == NULL) {
@@ -1378,14 +1479,18 @@ cm_certext_build_crldp(struct cm_store_entry *entry, PLArenaPool *arena,
 	if (values == NULL) {
 		return NULL;
 	}
-	for (i = 0; i < n; i++) {
-		location[i].type = certURI;
-		location[i].name.other.data = (unsigned char *) crldp[i];
-		location[i].name.other.len = strlen(crldp[i]);
-		location[i].l.next = &location[i].l;
-		value[i].distPointType = generalName;
-		value[i].distPoint.fullName = &location[i];
-		values[i] = &value[i];
+	for (i = 0, j = 0; i < n; i++) {
+		if (strlen(crldp[i]) == 0) {
+			continue;
+		}
+		location[j].type = certURI;
+		location[j].name.other.data = (unsigned char *) crldp[i];
+		location[j].name.other.len = strlen(crldp[i]);
+		location[j].l.next = &location[j].l;
+		value[j].distPointType = generalName;
+		value[j].distPoint.fullName = &location[j];
+		values[j] = &value[j];
+		j++;
 	}
 	decoded.distPoints = values;
 	memset(&encoded, 0, sizeof(encoded));
@@ -1405,6 +1510,9 @@ cm_certext_build_ns_comment(struct cm_store_entry *entry, PLArenaPool *arena,
 {
 	SECItem value, encoded, *item;
 
+	if (strlen(comment) == 0) {
+		return NULL;
+	}
 	memset(&value, 0, sizeof(value));
 	value.data = (unsigned char *) comment;
 	value.len = strlen(comment);
@@ -1418,6 +1526,26 @@ cm_certext_build_ns_comment(struct cm_store_entry *entry, PLArenaPool *arena,
 	return item;
 }
 
+/* Build a no-ocsp-checking extension value. */
+static SECItem *
+cm_certext_build_ocsp_no_check(struct cm_store_entry *entry,
+			       PLArenaPool *arena)
+{
+	SECItem value, encoded, *item;
+
+	memset(&value, 0, sizeof(value));
+	value.data = NULL;
+	value.len = 0;
+	memset(&encoded, 0, sizeof(encoded));
+	if (SEC_ASN1EncodeItem(arena, &encoded, &value,
+			       SEC_NullTemplate) == &encoded) {
+		item = SECITEM_ArenaDupItem(arena, &encoded);
+	} else {
+		item = NULL;
+	}
+	return item;
+}
+
 /* Build a requestedExtensions attribute. */
 void
 cm_certext_build_csr_extensions(struct cm_store_entry *entry,
@@ -1425,7 +1553,7 @@ cm_certext_build_csr_extensions(struct cm_store_entry *entry,
 				unsigned char **extensions, size_t *length)
 {
 	PLArenaPool *arena;
-	CERTCertExtension ext[9], *exts[10], **exts_ptr;
+	CERTCertExtension ext[11], *exts[12], **exts_ptr;
 	SECOidData *oid;
 	SECItem *item, encoded;
 	SECItem der_false = {
@@ -1437,9 +1565,10 @@ cm_certext_build_csr_extensions(struct cm_store_entry *entry,
 		.data = (unsigned char *) "\377",
 	};
 	int i;
-	char **tmp;
+	char **tmp, *comment;
 	const char *reason;
 	NSSInitContext *local_ctx = NULL;
+	const SEC_ASN1Template *template;
 
 	*extensions = NULL;
 	*length = 0;
@@ -1462,7 +1591,8 @@ cm_certext_build_csr_extensions(struct cm_store_entry *entry,
 		}
 		reason = util_n_fips_hook();
 		if (reason != NULL) {
-			cm_log(1, "Error putting NSS into FIPS mode: %s\n", reason);
+			cm_log(1, "Error putting NSS into FIPS mode: %s\n",
+			       reason);
 			return;
 		}
 	}
@@ -1483,7 +1613,8 @@ cm_certext_build_csr_extensions(struct cm_store_entry *entry,
 	item = cm_certext_build_san(entry, arena,
 				    entry->cm_template_hostname,
 				    entry->cm_template_email,
-				    entry->cm_template_principal);
+				    entry->cm_template_principal,
+				    entry->cm_template_ipaddress);
 	if (item != NULL) {
 		oid = SECOID_FindOIDByTag(SEC_OID_X509_SUBJECT_ALT_NAME);
 		if (oid != NULL) {
@@ -1564,10 +1695,33 @@ cm_certext_build_csr_extensions(struct cm_store_entry *entry,
 			i++;
 		}
 	}
+	if (entry->cm_template_freshest_crl != NULL) {
+		oid = SECOID_FindOIDByTag(SEC_OID_X509_FRESHEST_CRL);
+		tmp = entry->cm_template_freshest_crl;
+		item = cm_certext_build_crldp(entry, arena, tmp);
+		if ((item != NULL) && (oid != NULL)) {
+			ext[i].id = oid->oid;
+			ext[i].critical = der_false;
+			ext[i].value = *item;
+			exts[i] = &ext[i];
+			i++;
+		}
+	}
 	if (entry->cm_template_ns_comment != NULL) {
 		oid = SECOID_FindOIDByTag(SEC_OID_NS_CERT_EXT_COMMENT);
-		item = cm_certext_build_ns_comment(entry, arena,
-						   entry->cm_template_ns_comment);
+		comment = entry->cm_template_ns_comment;
+		item = cm_certext_build_ns_comment(entry, arena, comment);
+		if ((item != NULL) && (oid != NULL)) {
+			ext[i].id = oid->oid;
+			ext[i].critical = der_false;
+			ext[i].value = *item;
+			exts[i] = &ext[i];
+			i++;
+		}
+	}
+	if (entry->cm_template_no_ocsp_check) {
+		oid = SECOID_FindOIDByTag(SEC_OID_PKIX_OCSP_NO_CHECK);
+		item = cm_certext_build_ocsp_no_check(entry, arena);
 		if ((item != NULL) && (oid != NULL)) {
 			ext[i].id = oid->oid;
 			ext[i].critical = der_false;
@@ -1581,9 +1735,11 @@ cm_certext_build_csr_extensions(struct cm_store_entry *entry,
 	/* Encode the sequence. */
 	memset(&encoded, 0, sizeof(encoded));
 	if (i > 1) {
+		template = cm_certext_sequence_of_cert_extension_template;
 		if (SEC_ASN1EncodeItem(arena, &encoded, &exts_ptr,
-				       cm_certext_sequence_of_cert_extension_template) == &encoded) {
-			*extensions = talloc_memdup(entry, encoded.data, encoded.len);
+				       template) == &encoded) {
+			*extensions = talloc_memdup(entry, encoded.data,
+						    encoded.len);
 			if (*extensions != NULL) {
 				*length = encoded.len;
 			}
@@ -1687,14 +1843,14 @@ cm_certext_read_aia(struct cm_store_entry *entry, PLArenaPool *arena,
 }
 
 static void
-cm_certext_read_crldp(struct cm_store_entry *entry, PLArenaPool *arena,
-		      CERTCertExtension *ext)
+cm_certext_read_crlext(struct cm_store_entry *entry, PLArenaPool *arena,
+		       CERTCertExtension *ext, char ***dest)
 {
 	CERTCrlDistributionPoints *crldp;
 	CERTGeneralName *name;
 	SECItem uri;
 	void *parent;
-	char *tmp;
+	char *tmp, **list = *dest;
 	unsigned i, n;
 
 	crldp = CERT_DecodeCRLDistributionPoints(arena, &ext->value);
@@ -1710,11 +1866,10 @@ cm_certext_read_crldp(struct cm_store_entry *entry, PLArenaPool *arena,
 			}
 		}
 	}
-	talloc_free(entry->cm_cert_crl_distribution_point);
-	entry->cm_cert_crl_distribution_point = talloc_zero_array(entry,
-								  char *,
-								  n + 1);
-	if (entry->cm_cert_crl_distribution_point == NULL) {
+	talloc_free(list);
+	list = talloc_zero_array(entry, char *, n + 1);
+	if (list == NULL) {
+		*dest = list;
 		return;
 	}
 	for (i = 0, n = 0; crldp->distPoints[i] != NULL; i++) {
@@ -1723,14 +1878,30 @@ cm_certext_read_crldp(struct cm_store_entry *entry, PLArenaPool *arena,
 			name = crldp->distPoints[i]->distPoint.fullName;
 			if (name->type == certURI) {
 				uri = name->name.other;
-				parent = entry->cm_cert_crl_distribution_point;
+				parent = list;
 				tmp = talloc_strndup(parent,
 						     (char *) uri.data,
 						     uri.len);
-				entry->cm_cert_crl_distribution_point[n++] = tmp;
+				list[n++] = tmp;
 			}
 		}
 	}
+	*dest = list;
+}
+
+static void
+cm_certext_read_crldp(struct cm_store_entry *entry, PLArenaPool *arena,
+		      CERTCertExtension *ext)
+{
+	cm_certext_read_crlext(entry, arena, ext,
+			       &entry->cm_cert_crl_distribution_point);
+}
+
+static void
+cm_certext_read_freshest_crl(struct cm_store_entry *entry, PLArenaPool *arena,
+			     CERTCertExtension *ext)
+{
+	cm_certext_read_crlext(entry, arena, ext, &entry->cm_cert_freshest_crl);
 }
 
 static void
@@ -1764,8 +1935,9 @@ cm_certext_read_extensions(struct cm_store_entry *entry, PLArenaPool *arena,
 {
 	int i;
 	PLArenaPool *local_arena;
-	SECOidData *ku_oid, *eku_oid, *san_oid;
+	SECOidData *ku_oid, *eku_oid, *san_oid, *freshest_crl_oid;
 	SECOidData *basic_oid, *nsc_oid, *aia_oid, *crldp_oid, *profile_oid;
+	SECOidData *no_ocsp_check_oid;
 
 	if (extensions == NULL) {
 		return;
@@ -1821,7 +1993,20 @@ cm_certext_read_extensions(struct cm_store_entry *entry, PLArenaPool *arena,
 		       "extension.\n");
 		return;
 	}
+	freshest_crl_oid = SECOID_FindOIDByTag(SEC_OID_X509_FRESHEST_CRL);
+	if (freshest_crl_oid == NULL) {
+		cm_log(1, "Internal library error: unable to look up OID for "
+		       "freshest certificate revocation list extension.\n");
+		return;
+	}
+	no_ocsp_check_oid = SECOID_FindOIDByTag(SEC_OID_PKIX_OCSP_NO_CHECK);
+	if (no_ocsp_check_oid == NULL) {
+		cm_log(1, "Internal library error: unable to look up OID for "
+		       "no-OCSP-check extension.\n");
+		return;
+	}
 	profile_oid = (SECOidData *) &oid_microsoft_certtype;
+	entry->cm_cert_no_ocsp_check = FALSE;
 	for (i = 0; extensions[i] != NULL; i++) {
 		if (SECITEM_ItemsAreEqual(&ku_oid->oid, &extensions[i]->id)) {
 			cm_certext_read_ku(entry, arena, extensions[i]);
@@ -1832,7 +2017,8 @@ cm_certext_read_extensions(struct cm_store_entry *entry, PLArenaPool *arena,
 		if (SECITEM_ItemsAreEqual(&san_oid->oid, &extensions[i]->id)) {
 			cm_certext_read_san(entry, arena, extensions[i]);
 		}
-		if (SECITEM_ItemsAreEqual(&basic_oid->oid, &extensions[i]->id)) {
+		if (SECITEM_ItemsAreEqual(&basic_oid->oid,
+					  &extensions[i]->id)) {
 			cm_certext_read_basic(entry, arena, extensions[i]);
 		}
 		if (SECITEM_ItemsAreEqual(&nsc_oid->oid, &extensions[i]->id)) {
@@ -1841,12 +2027,22 @@ cm_certext_read_extensions(struct cm_store_entry *entry, PLArenaPool *arena,
 		if (SECITEM_ItemsAreEqual(&aia_oid->oid, &extensions[i]->id)) {
 			cm_certext_read_aia(entry, arena, extensions[i]);
 		}
-		if (SECITEM_ItemsAreEqual(&crldp_oid->oid, &extensions[i]->id)) {
+		if (SECITEM_ItemsAreEqual(&crldp_oid->oid,
+					  &extensions[i]->id)) {
 			cm_certext_read_crldp(entry, arena, extensions[i]);
 		}
-		if (SECITEM_ItemsAreEqual(&profile_oid->oid, &extensions[i]->id)) {
+		if (SECITEM_ItemsAreEqual(&freshest_crl_oid->oid,
+					  &extensions[i]->id)) {
+			cm_certext_read_freshest_crl(entry, arena, extensions[i]);
+		}
+		if (SECITEM_ItemsAreEqual(&profile_oid->oid,
+					  &extensions[i]->id)) {
 			cm_certext_read_profile(entry, arena, extensions[i]);
 		}
+		if (SECITEM_ItemsAreEqual(&no_ocsp_check_oid->oid,
+					  &extensions[i]->id)) {
+			entry->cm_cert_no_ocsp_check = TRUE;
+		}
 	}
 	if (arena == local_arena) {
 		PORT_FreeArena(local_arena, PR_TRUE);
