@@ -51,6 +51,7 @@
 
 struct cm_certread_state {
 	struct cm_certread_state_pvt pvt;
+	struct cm_store_entry *entry;
 	struct cm_subproc_state *subproc;
 };
 struct cm_certread_n_settings {
@@ -79,6 +80,17 @@ cm_certread_n_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 	FILE *fp;
 	const char *es;
 
+	if (entry->cm_cert_storage_location == NULL) {
+		cm_log(1, "Error reading certificate: no location "
+		       "specified.\n");
+		_exit(1);
+	}
+	if (entry->cm_cert_nickname == NULL) {
+		cm_log(1, "Error reading certificate: no nickname "
+		       "specified.\n");
+		_exit(1);
+	}
+
 	/* Open the status descriptor for stdio. */
 	fp = fdopen(fd, "w");
 	if (fp == NULL) {
@@ -143,12 +155,12 @@ cm_certread_n_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 			break;
 		}
 		cm_log(1, "Unable to open NSS database.\n");
-		_exit(1);
+		_exit(status);
 	}
 	es = util_n_fips_hook();
 	if (es != NULL) {
 		cm_log(1, "Error putting NSS into FIPS mode: %s\n", es);
-		_exit(1);
+		_exit(CM_SUB_STATUS_ERROR_INITIALIZING);
 	}
 	/* Allocate a memory pool. */
 	arena = PORT_NewArena(sizeof(double));
@@ -386,7 +398,7 @@ cm_certread_n_parse(struct cm_store_entry *entry,
 	/* Pick out the interesting bits. */
 	/* Issuer name */
 	talloc_free(entry->cm_cert_issuer_der);
-	entry->cm_cert_issuer_der = cm_store_hex_from_bin(NULL,
+	entry->cm_cert_issuer_der = cm_store_hex_from_bin(entry,
 							  cert->derIssuer.data,
 							  cert->derIssuer.len);
 	talloc_free(entry->cm_cert_issuer);
@@ -394,13 +406,13 @@ cm_certread_n_parse(struct cm_store_entry *entry,
 	/* Serial number */
 	talloc_free(entry->cm_cert_serial);
 	item = cert->serialNumber;
-	entry->cm_cert_serial = cm_store_hex_from_bin(NULL, item.data,
+	entry->cm_cert_serial = cm_store_hex_from_bin(entry, item.data,
 						      item.len);
 	/* Subject name */
 	talloc_free(entry->cm_cert_subject_der);
-	entry->cm_cert_subject_der = cm_store_hex_from_bin(NULL,
-							   cert->derSubject.data,
-							   cert->derSubject.len);
+	item = cert->derSubject;
+	entry->cm_cert_subject_der = cm_store_hex_from_bin(entry, item.data,
+							   item.len);
 	talloc_free(entry->cm_cert_subject);
 	entry->cm_cert_subject = talloc_strdup(entry, cert->subjectName);
 	/* Subject Public Key Info, encoded into a blob. */
@@ -415,7 +427,7 @@ cm_certread_n_parse(struct cm_store_entry *entry,
 		}
 		_exit(1);
 	}
-	entry->cm_cert_spki = cm_store_hex_from_bin(NULL, items->data,
+	entry->cm_cert_spki = cm_store_hex_from_bin(entry, items->data,
 						    items->len);
 	/* Not-before date. */
 	p = talloc_strndup(entry, (char *) cert->validity.notBefore.data,
@@ -442,6 +454,9 @@ cm_certread_n_parse(struct cm_store_entry *entry,
 	/* Principal name from subjectAltName extension. */
 	talloc_free(entry->cm_cert_principal);
 	entry->cm_cert_principal = NULL;
+	/* IP address from subjectAltName extension. */
+	talloc_free(entry->cm_cert_ipaddress);
+	entry->cm_cert_ipaddress = NULL;
 	/* Key usage from keyUsage extension. */
 	talloc_free(entry->cm_cert_ku);
 	entry->cm_cert_ku = NULL;
@@ -477,31 +492,28 @@ cm_certread_n_parse(struct cm_store_entry *entry,
 /* Check if something changed, for example we finished reading the data we need
  * from the cert. */
 static int
-cm_certread_n_ready(struct cm_store_entry *entry,
-		    struct cm_certread_state *state)
+cm_certread_n_ready(struct cm_certread_state *state)
 {
-	return cm_subproc_ready(entry, state->subproc);
+	return cm_subproc_ready(state->subproc);
 }
 
 /* Get a selectable-for-read descriptor we can poll for status changes. */
 static int
-cm_certread_n_get_fd(struct cm_store_entry *entry,
-		     struct cm_certread_state *state)
+cm_certread_n_get_fd(struct cm_certread_state *state)
 {
-	return cm_subproc_get_fd(entry, state->subproc);
+	return cm_subproc_get_fd(state->subproc);
 }
 
 /* Clean up after reading the certificate. */
 static void
-cm_certread_n_done(struct cm_store_entry *entry,
-		   struct cm_certread_state *state)
+cm_certread_n_done(struct cm_certread_state *state)
 {
+	const char *msg;
+
 	if (state->subproc != NULL) {
-		cm_certread_read_data_from_buffer(entry,
-						  cm_subproc_get_msg(entry,
-								     state->subproc,
-								     NULL));
-		cm_subproc_done(entry, state->subproc);
+		msg = cm_subproc_get_msg(state->subproc, NULL);
+		cm_certread_read_data_from_buffer(state->entry, msg);
+		cm_subproc_done(state->subproc);
 	}
 	talloc_free(state);
 }
@@ -525,7 +537,8 @@ cm_certread_n_start(struct cm_store_entry *entry)
 		state->pvt.ready = cm_certread_n_ready;
 		state->pvt.get_fd= cm_certread_n_get_fd;
 		state->pvt.done= cm_certread_n_done;
-		state->subproc = cm_subproc_start(cm_certread_n_main,
+		state->entry = entry;
+		state->subproc = cm_subproc_start(cm_certread_n_main, state,
 						  NULL, entry, &settings);
 		if (state->subproc == NULL) {
 			talloc_free(state);
