@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Red Hat, Inc.
+ * Copyright (C) 2009,2010,2012,2014 Red Hat, Inc.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 #include <sys/types.h>
 #include <stdio.h>
 #include <string.h>
+#include <time.h>
 
 #include <talloc.h>
 
@@ -60,30 +61,32 @@ cm_certread_start(struct cm_store_entry *entry)
 
 /* Check if something changed, for example we finished reading the cert. */
 int
-cm_certread_ready(struct cm_store_entry *entry, struct cm_certread_state *state)
+cm_certread_ready(struct cm_certread_state *state)
 {
 	struct cm_certread_state_pvt *pvt;
+
 	pvt = (struct cm_certread_state_pvt *) state;
-	return pvt->ready(entry, state);
+	return pvt->ready(state);
 }
 
 /* Get a selectable-for-read descriptor we can poll for status changes. */
 int
-cm_certread_get_fd(struct cm_store_entry *entry,
-		   struct cm_certread_state *state)
+cm_certread_get_fd(struct cm_certread_state *state)
 {
 	struct cm_certread_state_pvt *pvt;
+
 	pvt = (struct cm_certread_state_pvt *) state;
-	return pvt->get_fd(entry, state);
+	return pvt->get_fd(state);
 }
 
 /* Clean up after reading the certificate. */
 void
-cm_certread_done(struct cm_store_entry *entry, struct cm_certread_state *state)
+cm_certread_done(struct cm_certread_state *state)
 {
 	struct cm_certread_state_pvt *pvt;
+
 	pvt = (struct cm_certread_state_pvt *) state;
-	pvt->done(entry, state);
+	pvt->done(state);
 }
 
 /* Send what we know about this certificate down a pipe using stdio. */
@@ -91,21 +94,15 @@ void
 cm_certread_write_data_to_pipe(struct cm_store_entry *entry, FILE *fp)
 {
 	int i;
+	unsigned char *p;
+
 	fprintf(fp, " %s\n", entry->cm_cert_issuer_der ?: "");
-	fprintf(fp, " %s\n",
-		entry->cm_cert_issuer ?
-		cm_store_base64_from_bin(NULL,
-					 (unsigned char *) entry->cm_cert_issuer,
-					 -1) :
-		"");
+	p = (unsigned char *) entry->cm_cert_issuer;
+	fprintf(fp, " %s\n", p ? cm_store_base64_from_bin(NULL, p, -1) : "");
 	fprintf(fp, " %s\n", entry->cm_cert_serial ?: "");
 	fprintf(fp, " %s\n", entry->cm_cert_subject_der ?: "");
-	fprintf(fp, " %s\n",
-		entry->cm_cert_subject ?
-		cm_store_base64_from_bin(NULL,
-					 (unsigned char *) entry->cm_cert_subject,
-					 -1) :
-		"");
+	p = (unsigned char *) entry->cm_cert_subject;
+	fprintf(fp, " %s\n", p ? cm_store_base64_from_bin(NULL, p, -1) : "");
 	fprintf(fp, " %s\n", entry->cm_cert_spki ?: "");
 	fprintf(fp, " %lu\n", entry->cm_cert_not_before ?: 0);
 	fprintf(fp, " %lu\n", entry->cm_cert_not_after ?: 0);
@@ -113,40 +110,42 @@ cm_certread_write_data_to_pipe(struct cm_store_entry *entry, FILE *fp)
 	     (entry->cm_cert_hostname != NULL) &&
 	     (entry->cm_cert_hostname[i] != NULL);
 	     i++) {
+		p = (unsigned char *) entry->cm_cert_hostname[i];
 		fprintf(fp, "%s%s", (i > 0) ? "," : " ",
-			cm_store_base64_from_bin(NULL,
-						 (unsigned char *) entry->cm_cert_hostname[i],
-						 -1));
+			cm_store_base64_from_bin(NULL, p, -1));
 	}
 	fprintf(fp, "%s\n", i > 0 ? "" : " ");
 	for (i = 0;
 	     (entry->cm_cert_email != NULL) &&
 	     (entry->cm_cert_email[i] != NULL);
 	     i++) {
+		p = (unsigned char *) entry->cm_cert_email[i];
 		fprintf(fp, "%s%s", (i > 0) ? "," : " ",
-			cm_store_base64_from_bin(NULL,
-						 (unsigned char *) entry->cm_cert_email[i],
-						 -1));
+			cm_store_base64_from_bin(NULL, p, -1));
 	}
 	fprintf(fp, "%s\n", i > 0 ? "" : " ");
 	for (i = 0;
 	     (entry->cm_cert_principal != NULL) &&
 	     (entry->cm_cert_principal[i] != NULL);
 	     i++) {
+		p = (unsigned char *) entry->cm_cert_principal[i];
 		fprintf(fp, "%s%s", (i > 0) ? "," : " ",
-			cm_store_base64_from_bin(NULL,
-						 (unsigned char *) entry->cm_cert_principal[i],
-						 -1));
+			cm_store_base64_from_bin(NULL, p, -1));
+	}
+	fprintf(fp, "%s\n", i > 0 ? "" : " ");
+	for (i = 0;
+	     (entry->cm_cert_ipaddress != NULL) &&
+	     (entry->cm_cert_ipaddress[i] != NULL);
+	     i++) {
+		p = (unsigned char *) entry->cm_cert_ipaddress[i];
+		fprintf(fp, "%s%s", (i > 0) ? "," : " ",
+			cm_store_base64_from_bin(NULL, p, -1));
 	}
 	fprintf(fp, "%s\n", i > 0 ? "" : " ");
 	fprintf(fp, " %s\n", entry->cm_cert_ku ?: "");
 	fprintf(fp, " %s\n", entry->cm_cert_eku ?: "");
-	fprintf(fp, " %s\n",
-		entry->cm_cert_token ?
-		cm_store_base64_from_bin(NULL,
-					 (unsigned char *) entry->cm_cert_token,
-					 -1) :
-		"");
+	p = (unsigned char *) entry->cm_cert_token;
+	fprintf(fp, " %s\n", p ? cm_store_base64_from_bin(NULL, p, -1) : "");
 	fprintf(fp, " %d\n", entry->cm_cert_is_ca ? 1 : 0);
 	fprintf(fp, " %d\n", entry->cm_cert_is_ca ?
 		entry->cm_cert_ca_path_length : -1);
@@ -154,34 +153,34 @@ cm_certread_write_data_to_pipe(struct cm_store_entry *entry, FILE *fp)
 	     (entry->cm_cert_ocsp_location != NULL) &&
 	     (entry->cm_cert_ocsp_location[i] != NULL);
 	     i++) {
+		p = (unsigned char *) entry->cm_cert_ocsp_location[i];
 		fprintf(fp, "%s%s", (i > 0) ? "," : " ",
-			cm_store_base64_from_bin(NULL,
-						 (unsigned char *) entry->cm_cert_ocsp_location[i],
-						 -1));
+			cm_store_base64_from_bin(NULL, p, -1));
 	}
 	fprintf(fp, "%s\n", i > 0 ? "" : " ");
 	for (i = 0;
 	     (entry->cm_cert_crl_distribution_point != NULL) &&
 	     (entry->cm_cert_crl_distribution_point[i] != NULL);
 	     i++) {
+		p = (unsigned char *) entry->cm_cert_crl_distribution_point[i];
 		fprintf(fp, "%s%s", (i > 0) ? "," : " ",
-			cm_store_base64_from_bin(NULL,
-						 (unsigned char *) entry->cm_cert_crl_distribution_point[i],
-						 -1));
+			cm_store_base64_from_bin(NULL, p, -1));
 	}
 	fprintf(fp, "%s\n", i > 0 ? "" : " ");
-	fprintf(fp, " %s\n",
-		entry->cm_cert_ns_comment ?
-		cm_store_base64_from_bin(NULL,
-					 (unsigned char *) entry->cm_cert_ns_comment,
-					 -1) :
-		"");
-	fprintf(fp, " %s\n",
-		entry->cm_cert_profile ?
-		cm_store_base64_from_bin(NULL,
-					 (unsigned char *) entry->cm_cert_profile,
-					 -1) :
-		"");
+	for (i = 0;
+	     (entry->cm_cert_freshest_crl != NULL) &&
+	     (entry->cm_cert_freshest_crl[i] != NULL);
+	     i++) {
+		p = (unsigned char *) entry->cm_cert_freshest_crl[i];
+		fprintf(fp, "%s%s", (i > 0) ? "," : " ",
+			cm_store_base64_from_bin(NULL, p, -1));
+	}
+	fprintf(fp, "%s\n", i > 0 ? "" : " ");
+	p = (unsigned char *) entry->cm_cert_ns_comment;
+	fprintf(fp, " %s\n", p ? cm_store_base64_from_bin(NULL, p, -1) : "");
+	p = (unsigned char *) entry->cm_cert_profile;
+	fprintf(fp, " %s\n", p ? cm_store_base64_from_bin(NULL, p, -1) : "");
+	fprintf(fp, " %d\n", entry->cm_cert_no_ocsp_check ? 1 : 0);
 	fprintf(fp, " %s\n", entry->cm_cert ?: "");
 }
 
@@ -229,9 +228,9 @@ cm_certread_read_data_from_buffer(struct cm_store_entry *entry, const char *p)
 			talloc_free(entry->cm_cert_subject);
 			entry->cm_cert_subject = (p == q) ? NULL :
 						 cm_store_base64_as_bin(entry,
-								        p,
-								        q - p,
-								        NULL);
+									p,
+									q - p,
+									NULL);
 			break;
 		case 5:
 			talloc_free(entry->cm_cert_spki);
@@ -259,10 +258,10 @@ cm_certread_read_data_from_buffer(struct cm_store_entry *entry, const char *p)
 			while ((*u != '\0') && (u < q)) {
 				v = u + strcspn(u, ",\r\n");
 				if (v > u) {
-					entry->cm_cert_hostname[j] = cm_store_base64_as_bin(vals,
-											    u,
-											    v - u,
-											    NULL);
+					entry->cm_cert_hostname[j] =
+						cm_store_base64_as_bin(vals, u,
+								       v - u,
+								       NULL);
 					j++;
 				}
 				u = v + strspn(u, ",\r\n");
@@ -279,10 +278,10 @@ cm_certread_read_data_from_buffer(struct cm_store_entry *entry, const char *p)
 			while ((*u != '\0') && (u < q)) {
 				v = u + strcspn(u, ",\r\n");
 				if (v > u) {
-					entry->cm_cert_email[j] = cm_store_base64_as_bin(vals,
-											 u,
-											 v - u,
-											 NULL);
+					entry->cm_cert_email[j] =
+						cm_store_base64_as_bin(vals, u,
+								       v - u,
+								       NULL);
 					j++;
 				}
 				u = v + strspn(u, ",\r\n");
@@ -299,41 +298,58 @@ cm_certread_read_data_from_buffer(struct cm_store_entry *entry, const char *p)
 			while ((*u != '\0') && (u < q)) {
 				v = u + strcspn(u, ",\r\n");
 				if (v > u) {
-					entry->cm_cert_principal[j] = cm_store_base64_as_bin(vals,
-											     u,
-											     v - u,
-											     NULL);
+					entry->cm_cert_principal[j] =
+						cm_store_base64_as_bin(vals, u,
+								       v - u,
+								       NULL);
 					j++;
 				}
 				u = v + strspn(u, ",\r\n");
 			}
 			break;
 		case 11:
+			talloc_free(entry->cm_cert_ipaddress);
+			entry->cm_cert_ipaddress = talloc_zero_array(entry,
+								     char *,
+								     q - p + 2);
+			vals = entry->cm_cert_ipaddress;
+			u = p;
+			j = 0;
+			while ((*u != '\0') && (u < q)) {
+				v = u + strcspn(u, ",\r\n");
+				if (v > u) {
+					entry->cm_cert_ipaddress[j] =
+						talloc_strndup(vals, u, v - u);
+					j++;
+				}
+				u = v + strspn(u, ",\r\n");
+			}
+			break;
+		case 12:
 			talloc_free(entry->cm_cert_ku);
 			entry->cm_cert_ku = (p == q) ? NULL :
 					    talloc_strndup(entry, p, q - p);
 			break;
-		case 12:
+		case 13:
 			talloc_free(entry->cm_cert_eku);
 			entry->cm_cert_eku = (p == q) ? NULL :
 					     talloc_strndup(entry, p, q - p);
 			break;
-		case 13:
+		case 14:
 			if (p != q) {
 				talloc_free(entry->cm_cert_token);
-				entry->cm_cert_token = cm_store_base64_as_bin(entry,
-									      p,
-									      q - p,
-									      NULL);
+				entry->cm_cert_token =
+					cm_store_base64_as_bin(entry, p, q - p,
+							       NULL);
 			}
 			break;
-		case 14:
+		case 15:
 			entry->cm_cert_is_ca = (p != q) ? (atoi(p) != 0) : 0;
 			break;
-		case 15:
+		case 16:
 			entry->cm_cert_ca_path_length = (p != q) ? atoi(p) : -1;
 			break;
-		case 16:
+		case 17:
 			talloc_free(entry->cm_cert_ocsp_location);
 			entry->cm_cert_ocsp_location = talloc_zero_array(entry,
 									 char *,
@@ -344,20 +360,19 @@ cm_certread_read_data_from_buffer(struct cm_store_entry *entry, const char *p)
 			while ((*u != '\0') && (u < q)) {
 				v = u + strcspn(u, ",\r\n");
 				if (v > u) {
-					entry->cm_cert_ocsp_location[j] = cm_store_base64_as_bin(vals,
-												 u,
-												 v - u,
-												 NULL);
+					entry->cm_cert_ocsp_location[j] =
+						cm_store_base64_as_bin(vals, u,
+								       v - u,
+								       NULL);
 					j++;
 				}
 				u = v + strspn(u, ",\r\n");
 			}
 			break;
-		case 17:
+		case 18:
 			talloc_free(entry->cm_cert_crl_distribution_point);
-			entry->cm_cert_crl_distribution_point = talloc_zero_array(entry,
-										  char *,
-										  q - p + 2);
+			entry->cm_cert_crl_distribution_point =
+				talloc_zero_array(entry, char *, q - p + 2);
 			vals = entry->cm_cert_crl_distribution_point;
 			u = p;
 			j = 0;
@@ -373,23 +388,46 @@ cm_certread_read_data_from_buffer(struct cm_store_entry *entry, const char *p)
 				u = v + strspn(u, ",\r\n");
 			}
 			break;
-		case 18:
+		case 19:
+			talloc_free(entry->cm_cert_freshest_crl);
+			entry->cm_cert_freshest_crl = talloc_zero_array(entry,
+									char *,
+									q - p + 2);
+			vals = entry->cm_cert_freshest_crl;
+			u = p;
+			j = 0;
+			while ((*u != '\0') && (u < q)) {
+				v = u + strcspn(u, ",\r\n");
+				if (v > u) {
+					entry->cm_cert_freshest_crl[j] = cm_store_base64_as_bin(vals,
+												u,
+												v - u,
+												NULL);
+					j++;
+				}
+				u = v + strspn(u, ",\r\n");
+			}
+			break;
+		case 20:
 			talloc_free(entry->cm_cert_ns_comment);
 			entry->cm_cert_ns_comment = (p == q) ? NULL :
 						    cm_store_base64_as_bin(entry,
-								           p,
-								           q - p,
-								           NULL);
+									   p,
+									   q - p,
+									   NULL);
 			break;
-		case 19:
+		case 21:
 			talloc_free(entry->cm_cert_profile);
 			entry->cm_cert_profile = (p == q) ? NULL :
 						 cm_store_base64_as_bin(entry,
-								        p,
-								        q - p,
-								        NULL);
+									p,
+									q - p,
+									NULL);
 			break;
-		case 20:
+		case 22:
+			entry->cm_cert_no_ocsp_check = (p != q) ? (atoi(p) != 0) : 0;
+			break;
+		case 23:
 			talloc_free(entry->cm_cert);
 			entry->cm_cert = (p[strspn(p, " \r\n")] == '\0') ?
 					 NULL :
