@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009,2013 Red Hat, Inc.
+ * Copyright (C) 2009,2013,2014 Red Hat, Inc.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,31 +27,24 @@ struct cm_certsave_state *cm_certsave_n_start(struct cm_store_entry *entry);
 struct cm_certsave_state *cm_certsave_o_start(struct cm_store_entry *entry);
 
 /* Check if something changed, for example we finished saving the cert. */
-int cm_certsave_ready(struct cm_store_entry *entry,
-		      struct cm_certsave_state *state);
+int cm_certsave_ready(struct cm_certsave_state *state);
 
 /* Get a selectable-for-read descriptor we can poll for status changes. */
-int cm_certsave_get_fd(struct cm_store_entry *entry,
-		       struct cm_certsave_state *state);
+int cm_certsave_get_fd(struct cm_certsave_state *state);
 
 /* Check if we saved the certificate. */
-int cm_certsave_saved(struct cm_store_entry *entry,
-		      struct cm_certsave_state *state);
+int cm_certsave_saved(struct cm_certsave_state *state);
 
 /* Check if we failed due to a subject name conflict. */
-int cm_certsave_conflict_subject(struct cm_store_entry *entry,
-				 struct cm_certsave_state *state);
+int cm_certsave_conflict_subject(struct cm_certsave_state *state);
 
 /* Check if we failed due to a nickname conflict. */
-int cm_certsave_conflict_nickname(struct cm_store_entry *entry,
-				  struct cm_certsave_state *state);
+int cm_certsave_conflict_nickname(struct cm_certsave_state *state);
 
 /* Check if we failed due to a permissions error. */
-int cm_certsave_permissions_error(struct cm_store_entry *entry,
-				  struct cm_certsave_state *state);
+int cm_certsave_permissions_error(struct cm_certsave_state *state);
 
 /* Clean up after saving the certificate. */
-void cm_certsave_done(struct cm_store_entry *entry,
-		      struct cm_certsave_state *state);
+void cm_certsave_done(struct cm_certsave_state *state);
 
 #endif
