@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Red Hat, Inc.
+ * Copyright (C) 2009,2014 Red Hat, Inc.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,25 +20,19 @@
 
 struct cm_csrgen_state_pvt {
 	/* Check if a CSR is ready. */
-	int (*ready)(struct cm_store_entry *entry,
-		     struct cm_csrgen_state *state);
-	/* Get a selectable-for-read descriptor we can poll for status changes.
-	 */
-	int (*get_fd)(struct cm_store_entry *entry,
-		      struct cm_csrgen_state *state);
+	int (*ready)(struct cm_csrgen_state *state);
+	/* Get a selectable-for-read descriptor which will either have data or
+	 * be closed when status changes. */
+	int (*get_fd)(struct cm_csrgen_state *state);
 	/* Save the CSR to the entry. */
-	int (*save_csr)(struct cm_store_entry *entry,
-		        struct cm_csrgen_state *state);
+	int (*save_csr)(struct cm_csrgen_state *state);
 	/* Check if we need a PIN (or a new PIN) to get at the key material. */
-	int (*need_pin)(struct cm_store_entry *entry,
-		        struct cm_csrgen_state *state);
+	int (*need_pin)(struct cm_csrgen_state *state);
 	/* Check if we need the token to be inserted to get at the key
 	 * material. */
-	int (*need_token)(struct cm_store_entry *entry,
-			  struct cm_csrgen_state *state);
+	int (*need_token)(struct cm_csrgen_state *state);
 	/* Clean up after CSR generation. */
-	void (*done)(struct cm_store_entry *entry,
-		     struct cm_csrgen_state *state);
+	void (*done)(struct cm_csrgen_state *state);
 };
 
 #endif
