@@ -23,6 +23,7 @@
 #include <fcntl.h>
 #include <stdlib.h>
 #include <string.h>
+#include <time.h>
 #include <unistd.h>
 
 #include <prerror.h>
@@ -54,6 +55,7 @@
 
 struct cm_csrgen_state {
 	struct cm_csrgen_state_pvt pvt;
+	struct cm_store_entry *entry;
 	struct cm_subproc_state *subproc;
 };
 
@@ -148,7 +150,7 @@ cm_csrgen_n_attributes(struct cm_store_entry *entry, NSSInitContext *ctx,
 							 &bmp.data,
 							 &bmp.len) == 0) &&
 			    (SEC_ASN1EncodeItem(arena, &friendly, &bmp,
-					        SEC_BMPStringTemplate) == &friendly)) {
+						SEC_BMPStringTemplate) == &friendly)) {
 				friendlies[0] = &friendly;
 				friendlies[1] = NULL;
 				attr[i].attrType = oid->oid;
@@ -303,7 +305,7 @@ cm_csrgen_n_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 			memset(&utf8, 0, sizeof(utf8));
 			if (SEC_ASN1EncodeItem(arena, &utf8, &item,
 					       SEC_PrintableStringTemplate) == &utf8) {
-				q = cm_store_hex_from_bin(NULL,
+				q = cm_store_hex_from_bin(entry,
 							  utf8.data,
 							  utf8.len);
 				if (q != NULL) {
@@ -621,45 +623,44 @@ cm_csrgen_n_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 
 /* Check if a CSR is ready. */
 static int
-cm_csrgen_n_ready(struct cm_store_entry *entry, struct cm_csrgen_state *state)
+cm_csrgen_n_ready(struct cm_csrgen_state *state)
 {
-	return cm_subproc_ready(entry, state->subproc);
+	return cm_subproc_ready(state->subproc);
 }
 
 /* Get a selectable-for-read descriptor we can poll for status changes. */
 static int
-cm_csrgen_n_get_fd(struct cm_store_entry *entry, struct cm_csrgen_state *state)
+cm_csrgen_n_get_fd(struct cm_csrgen_state *state)
 {
-	return cm_subproc_get_fd(entry, state->subproc);
+	return cm_subproc_get_fd(state->subproc);
 }
 
 /* Save the CSR to the entry. */
 static int
-cm_csrgen_n_save_csr(struct cm_store_entry *entry,
-		     struct cm_csrgen_state *state)
+cm_csrgen_n_save_csr(struct cm_csrgen_state *state)
 {
 	int status;
 	char *p, *q;
-	status = cm_subproc_get_exitstatus(entry, state->subproc);
+
+	status = cm_subproc_get_exitstatus(state->subproc);
 	if (!WIFEXITED(status) || (WEXITSTATUS(status) != 0)) {
 		return -1;
 	}
-	talloc_free(entry->cm_csr);
-	entry->cm_csr = talloc_strdup(entry,
-				      cm_subproc_get_msg(entry,
-							 state->subproc,
-							 NULL));
-	if (entry->cm_csr == NULL) {
+	talloc_free(state->entry->cm_csr);
+	state->entry->cm_csr =
+		talloc_strdup(state->entry,
+			      cm_subproc_get_msg(state->subproc, NULL));
+	if (state->entry->cm_csr == NULL) {
 		return ENOMEM;
 	}
-	p = strstr(entry->cm_csr, "-----END");
+	p = strstr(state->entry->cm_csr, "-----END");
 	if (p != NULL) {
 		p = strstr(p, "REQUEST-----");
 		if (p != NULL) {
 			p += strcspn(p, "\r\n");
 			q = p + strspn(p, "\r\n");
-			entry->cm_spkac = talloc_strdup(entry, q);
-			if (entry->cm_spkac == NULL) {
+			state->entry->cm_spkac = talloc_strdup(state->entry, q);
+			if (state->entry->cm_spkac == NULL) {
 				return ENOMEM;
 			}
 			*q = '\0';
@@ -670,11 +671,10 @@ cm_csrgen_n_save_csr(struct cm_store_entry *entry,
 
 /* Check if we need a PIN (or a new PIN) to access the key information. */
 static int
-cm_csrgen_n_need_pin(struct cm_store_entry *entry,
-		     struct cm_csrgen_state *state)
+cm_csrgen_n_need_pin(struct cm_csrgen_state *state)
 {
 	int status;
-	status = cm_subproc_get_exitstatus(entry, state->subproc);
+	status = cm_subproc_get_exitstatus(state->subproc);
 	if (WIFEXITED(status) &&
 	    (WEXITSTATUS(status) == CM_SUB_STATUS_ERROR_AUTH)) {
 		return 0;
@@ -684,11 +684,10 @@ cm_csrgen_n_need_pin(struct cm_store_entry *entry,
 
 /* Check if we need a token to be inserted to access the key information. */
 static int
-cm_csrgen_n_need_token(struct cm_store_entry *entry,
-		       struct cm_csrgen_state *state)
+cm_csrgen_n_need_token(struct cm_csrgen_state *state)
 {
 	int status;
-	status = cm_subproc_get_exitstatus(entry, state->subproc);
+	status = cm_subproc_get_exitstatus(state->subproc);
 	if (WIFEXITED(status) &&
 	    (WEXITSTATUS(status) == CM_SUB_STATUS_ERROR_NO_TOKEN)) {
 		return 0;
@@ -698,10 +697,10 @@ cm_csrgen_n_need_token(struct cm_store_entry *entry,
 
 /* Clean up after CSR generation. */
 static void
-cm_csrgen_n_done(struct cm_store_entry *entry, struct cm_csrgen_state *state)
+cm_csrgen_n_done(struct cm_csrgen_state *state)
 {
 	if (state->subproc != NULL) {
-		cm_subproc_done(entry, state->subproc);
+		cm_subproc_done(state->subproc);
 	}
 	talloc_free(state);
 }
@@ -720,7 +719,8 @@ cm_csrgen_n_start(struct cm_store_entry *entry)
 		state->pvt.need_pin = &cm_csrgen_n_need_pin;
 		state->pvt.need_token = &cm_csrgen_n_need_token;
 		state->pvt.done = &cm_csrgen_n_done;
-		state->subproc = cm_subproc_start(cm_csrgen_n_main,
+		state->entry = entry;
+		state->subproc = cm_subproc_start(cm_csrgen_n_main, state,
 						  NULL, entry, NULL);
 		if (state->subproc == NULL) {
 			talloc_free(state);
