@@ -18,7 +18,9 @@
 #include "config.h"
 
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <errno.h>
+#include <fcntl.h>
 #include <limits.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -28,21 +30,23 @@
 
 #include <krb5.h>
 
-#include <openssl/bn.h>
-#include <openssl/pem.h>
-#include <openssl/x509.h>
+#include <nss.h>
+#include <cert.h>
 
 #include <dbus/dbus.h>
 
 #include <talloc.h>
 
+#include "log.h"
 #include "prefs.h"
+#include "store.h"
 #include "submit-d.h"
 #include "submit-e.h"
 #include "submit-h.h"
 #include "submit-u.h"
 #include "util.h"
-#include "util-o.h"
+#include "util-m.h"
+#include "util-n.h"
 
 #ifdef ENABLE_NLS
 #include <libintl.h>
@@ -110,24 +114,22 @@ statevar(const char *state, const char *what)
 static char *
 serial_hex_from_cert(const char *cert)
 {
-	X509 *c;
-	BIGNUM *bn;
-	BIO *mem;
+	CERTCertificate *c;
+	char *ret = NULL, *pem;
 
 	if ((cert != NULL) && (strlen(cert) > 0)) {
-		mem = BIO_new_mem_buf((void *) cert, -1);
-		if (mem != NULL) {
-			c = PEM_read_bio_X509(mem, NULL, NULL, NULL);
+		pem = talloc_strdup(NULL, cert);
+		if (pem != NULL) {
+			c = CERT_DecodeCertFromPackage(pem, strlen(pem));
 			if (c != NULL) {
-				bn = ASN1_INTEGER_to_BN(X509_get_serialNumber(c),
-							NULL);
-				if (bn != NULL) {
-					return BN_bn2hex(bn);
-				}
+				ret = cm_store_hex_from_bin(NULL,
+							    c->serialNumber.data,
+							    c->serialNumber.len);
+				CERT_DestroyCertificate(c);
 			}
 		}
 	}
-	return NULL;
+	return ret;
 }
 
 int
@@ -151,10 +153,29 @@ main(int argc, char **argv)
 	dbus_bool_t can_agent, use_agent, missing_args = FALSE;
 	struct dogtag_default **defaults;
 	enum cm_external_status ret;
+	NSSInitContext *nctx;
+	const char *es;
+	const char *mode = CM_OP_SUBMIT;
+
+	if (getenv(CM_SUBMIT_OPERATION_ENV) != NULL) {
+		mode = getenv(CM_SUBMIT_OPERATION_ENV);
+	}
+	if ((strcasecmp(mode, CM_OP_SUBMIT) == 0) ||
+	    (strcasecmp(mode, CM_OP_POLL) == 0)) {
+		/* fall through */
+	} else
+	if (strcasecmp(mode, CM_OP_IDENTIFY) == 0) {
+		printf("Dogtag (%s %s)\n", PACKAGE_NAME, PACKAGE_VERSION);
+		return 0;
+	} else {
+		/* unsupported request */
+		return CM_SUBMIT_STATUS_OPERATION_NOT_SUPPORTED;
+	}
 
 #ifdef ENABLE_NLS
 	bindtextdomain(PACKAGE, MYLOCALEDIR);
 #endif
+
 	savedstate = getenv(CM_SUBMIT_COOKIE_ENV);
 
 	while ((c = getopt(argc, argv, "E:A:d:n:i:C:c:k:p:P:s:D:S:T:vV:NR")) != -1) {
@@ -191,7 +212,7 @@ main(int argc, char **argv)
 			serial = optarg;
 			break;
 		case 's':
-			serial = util_o_dec_from_hex(optarg);
+			serial = util_dec_from_hex(optarg);
 			break;
 		case 'S':
 			savedstate = optarg;
@@ -220,6 +241,24 @@ main(int argc, char **argv)
 		}
 	}
 
+	umask(S_IRWXG | S_IRWXO);
+
+	nctx = NSS_InitContext(CM_DEFAULT_CERT_STORAGE_LOCATION,
+			       NULL, NULL, NULL, NULL,
+			       NSS_INIT_NOCERTDB |
+			       NSS_INIT_READONLY |
+			       NSS_INIT_NOROOTINIT |
+			       NSS_INIT_NOMODDB);
+	if (nctx == NULL) {
+		cm_log(1, "Unable to initialize NSS.\n");
+		_exit(1);
+	}
+	es = util_n_fips_hook();
+	if (es != NULL) {
+		cm_log(1, "Error putting NSS into FIPS mode: %s\n", es);
+		_exit(1);
+	}
+
 	ctx = talloc_new(NULL);
 
 	ipaconfig = read_config_file(IPACONFIG);
@@ -279,7 +318,7 @@ main(int argc, char **argv)
 			if (cm_prefs_dogtag_renew()) {
 				serial = serial_hex_from_cert(tmp);
 				if (serial != NULL) {
-					serial = util_o_dec_from_hex(serial);
+					serial = util_dec_from_hex(serial);
 				}
 			}
 		}
@@ -339,6 +378,10 @@ main(int argc, char **argv)
 		help(argv[0]);
 		return CM_SUBMIT_STATUS_UNCONFIGURED;
 	}
+	if (NSS_ShutdownContext(nctx) != SECSuccess) {
+		printf(_("Error shutting down NSS.\n"));
+		return CM_SUBMIT_STATUS_UNREACHABLE;
+	}
 
 	/* Figure out where we are in the multi-step process. */
 	op = op_none;
