@@ -23,6 +23,7 @@
 #include <fcntl.h>
 #include <limits.h>
 #include <stdlib.h>
+#include <time.h>
 #include <unistd.h>
 
 #include <nss.h>
@@ -53,6 +54,7 @@
 
 struct cm_keygen_state {
 	struct cm_keygen_state_pvt pvt;
+	struct cm_store_entry *entry;
 	struct cm_subproc_state *subproc;
 };
 
@@ -119,6 +121,13 @@ retry_gen:
 			_exit(CM_SUB_STATUS_INTERNAL_ERROR);
 		}
 		if (RSA_generate_key_ex(rsa, cm_key_size, exponent, NULL) != 1) {
+			if (cm_key_size != CM_DEFAULT_PUBKEY_SIZE) {
+				cm_log(1, "Error generating %d-bit key, "
+				       "attempting %d bits.\n",
+				       cm_key_size, CM_DEFAULT_PUBKEY_SIZE);
+				cm_key_size = CM_DEFAULT_PUBKEY_SIZE;
+				goto retry_gen;
+			}
 			cm_log(1, "Error generating key.\n");
 			_exit(CM_SUB_STATUS_INTERNAL_ERROR);
 		}
@@ -138,6 +147,13 @@ retry_gen:
 		if (DSA_generate_parameters_ex(dsa, cm_key_size,
 					       NULL, 0,
 					       NULL, NULL, NULL) != 1) {
+			if (cm_key_size != CM_DEFAULT_PUBKEY_SIZE) {
+				cm_log(1, "Error generating %d-bit key, "
+				       "attempting %d bits.\n",
+				       cm_key_size, CM_DEFAULT_PUBKEY_SIZE);
+				cm_key_size = CM_DEFAULT_PUBKEY_SIZE;
+				goto retry_gen;
+			}
 			cm_log(1, "Error generating parameters.\n");
 			_exit(CM_SUB_STATUS_INTERNAL_ERROR);
 		}
@@ -158,15 +174,15 @@ retry_gen:
 			ecurve = NID_secp521r1;
 		ec = EC_KEY_new_by_curve_name(ecurve);
 		while ((ec == NULL) && (ecurve != NID_X9_62_prime256v1)) {
-			cm_log(1, "Error allocating new EC key.");
+			cm_log(1, "Error allocating new EC key.\n");
 			switch (ecurve) {
 			case NID_secp521r1:
-				cm_log(1, "Trying with a smaller key.");
+				cm_log(1, "Trying with a smaller key.\n");
 				ecurve = NID_secp384r1;
 				ec = EC_KEY_new_by_curve_name(ecurve);
 				break;
 			case NID_secp384r1:
-				cm_log(1, "Trying with a smaller key.");
+				cm_log(1, "Trying with a smaller key.\n");
 				ecurve = NID_X9_62_prime256v1;
 				ec = EC_KEY_new_by_curve_name(ecurve);
 				break;
@@ -273,25 +289,24 @@ retry_gen:
 
 /* Check if the keypair is ready. */
 static int
-cm_keygen_o_ready(struct cm_store_entry *entry, struct cm_keygen_state *state)
+cm_keygen_o_ready(struct cm_keygen_state *state)
 {
-	return cm_subproc_ready(entry, state->subproc);
+	return cm_subproc_ready(state->subproc);
 }
 
 /* Get a selectable-for-read descriptor we can poll for status changes. */
 static int
-cm_keygen_o_get_fd(struct cm_store_entry *entry, struct cm_keygen_state *state)
+cm_keygen_o_get_fd(struct cm_keygen_state *state)
 {
-	return cm_subproc_get_fd(entry, state->subproc);
+	return cm_subproc_get_fd(state->subproc);
 }
 
 /* Tell us if the keypair was saved to the location specified in the entry. */
 static int
-cm_keygen_o_saved_keypair(struct cm_store_entry *entry,
-		          struct cm_keygen_state *state)
+cm_keygen_o_saved_keypair(struct cm_keygen_state *state)
 {
 	int status;
-	status = cm_subproc_get_exitstatus(entry, state->subproc);
+	status = cm_subproc_get_exitstatus(state->subproc);
 	if (WIFEXITED(status) && (WEXITSTATUS(status) == 0)) {
 		return 0;
 	}
@@ -300,11 +315,10 @@ cm_keygen_o_saved_keypair(struct cm_store_entry *entry,
 
 /* Tell us if we don't have permissions. */
 static int
-cm_keygen_o_need_perms(struct cm_store_entry *entry,
-		       struct cm_keygen_state *state)
+cm_keygen_o_need_perms(struct cm_keygen_state *state)
 {
 	int status;
-	status = cm_subproc_get_exitstatus(entry, state->subproc);
+	status = cm_subproc_get_exitstatus(state->subproc);
 	if (WIFEXITED(status) &&
 	    (WEXITSTATUS(status) == CM_SUB_STATUS_ERROR_PERMS)) {
 		return 0;
@@ -314,11 +328,10 @@ cm_keygen_o_need_perms(struct cm_store_entry *entry,
 
 /* Tell us if we need a new/correct PIN to use the key store. */
 static int
-cm_keygen_o_need_pin(struct cm_store_entry *entry,
-		     struct cm_keygen_state *state)
+cm_keygen_o_need_pin(struct cm_keygen_state *state)
 {
 	int status;
-	status = cm_subproc_get_exitstatus(entry, state->subproc);
+	status = cm_subproc_get_exitstatus(state->subproc);
 	if (WIFEXITED(status) &&
 	    (WEXITSTATUS(status) == CM_SUB_STATUS_ERROR_AUTH)) {
 		return 0;
@@ -328,11 +341,10 @@ cm_keygen_o_need_pin(struct cm_store_entry *entry,
 
 /* Check if we need a token to be inserted to generate the key. */
 static int
-cm_keygen_o_need_token(struct cm_store_entry *entry,
-		       struct cm_keygen_state *state)
+cm_keygen_o_need_token(struct cm_keygen_state *state)
 {
 	int status;
-	status = cm_subproc_get_exitstatus(entry, state->subproc);
+	status = cm_subproc_get_exitstatus(state->subproc);
 	if (WIFEXITED(status) &&
 	    (WEXITSTATUS(status) == CM_SUB_STATUS_ERROR_NO_TOKEN)) {
 		return 0;
@@ -342,28 +354,27 @@ cm_keygen_o_need_token(struct cm_store_entry *entry,
 
 /* Clean up after key generation. */
 static void
-cm_keygen_o_done(struct cm_store_entry *entry, struct cm_keygen_state *state)
+cm_keygen_o_done(struct cm_keygen_state *state)
 {
 	const char *pubkey_info, *p;
 	int len;
 
 	if (state->subproc != NULL) {
-		pubkey_info = cm_subproc_get_msg(entry, state->subproc, NULL);
+		pubkey_info = cm_subproc_get_msg(state->subproc, NULL);
 		if (pubkey_info != NULL) {
 			len = strcspn(pubkey_info, "\r\n");
-			entry->cm_key_pubkey_info = talloc_strndup(entry,
-								   pubkey_info,
-								   len);
+			state->entry->cm_key_pubkey_info =
+				talloc_strndup(state->entry, pubkey_info, len);
 			p = pubkey_info + len;
 			p += strspn(p, "\r\n");
 			len = strcspn(p, "\r\n");
-			entry->cm_key_pubkey = talloc_strndup(entry,
-							      p, len);
+			state->entry->cm_key_pubkey =
+				talloc_strndup(state->entry, p, len);
 		} else {
-			entry->cm_key_pubkey_info = NULL;
-			entry->cm_key_pubkey = NULL;
+			state->entry->cm_key_pubkey_info = NULL;
+			state->entry->cm_key_pubkey = NULL;
 		}
-		cm_subproc_done(entry, state->subproc);
+		cm_subproc_done(state->subproc);
 	}
 	talloc_free(state);
 }
@@ -386,7 +397,8 @@ cm_keygen_o_start(struct cm_store_entry *entry)
 		state->pvt.need_pin = cm_keygen_o_need_pin;
 		state->pvt.need_token = cm_keygen_o_need_token;
 		state->pvt.done = cm_keygen_o_done;
-		state->subproc = cm_subproc_start(cm_keygen_o_main,
+		state->entry = entry;
+		state->subproc = cm_subproc_start(cm_keygen_o_main, state,
 						  NULL, entry, NULL);
 		if (state->subproc == NULL) {
 			talloc_free(state);
