@@ -24,6 +24,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <syslog.h>
+#include <time.h>
 #include <unistd.h>
 
 #include <talloc.h>
@@ -252,6 +253,41 @@ cm_notify_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 			break;
 		}
 		break;
+	case cm_notify_event_ca_not_saved:
+		switch (entry->cm_cert_storage_type) {
+		case cm_cert_storage_nssdb:
+			if (entry->cm_cert_token != NULL) {
+				message = talloc_asprintf(entry, "CA certificate "
+							  "for certificate "
+							  "named \"%s\" "
+							  "in token \"%s\" "
+							  "in database \"%s\" "
+							  "(CA \"%s\") not saved.",
+							  entry->cm_cert_nickname,
+							  entry->cm_cert_token,
+							  entry->cm_cert_storage_location,
+							  entry->cm_ca_nickname);
+			} else {
+				message = talloc_asprintf(entry, "CA certificate "
+							  "for certificate "
+							  "named \"%s\" "
+							  "in database \"%s\" "
+							  "(CA \"%s\") not saved.",
+							  entry->cm_cert_nickname,
+							  entry->cm_cert_storage_location,
+							  entry->cm_ca_nickname);
+			}
+			break;
+		case cm_cert_storage_file:
+			message = talloc_asprintf(entry, "CA certificate "
+						  "for certificate "
+						  "in file \"%s\" "
+						  "(CA \"%s\") not saved.",
+						  entry->cm_cert_storage_location,
+						  entry->cm_ca_nickname);
+			break;
+		}
+		break;
 	}
 	method = entry->cm_notification_method;
 	if (method == cm_notification_unspecified) {
@@ -319,7 +355,7 @@ cm_notify_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 			return -1;
 		}
 		cm_log(1, "Running notification helper \"%s\".\n", argv[0]);
-		cm_subproc_mark_most_cloexec(entry, -1);
+		cm_subproc_mark_most_cloexec(-1);
 		setenv(CM_NOTIFICATION_ENV, message, 1);
 		if (execvp(argv[0], argv) == -1) {
 			cm_log(0, "Error execvp()ing command \"%s\" (\"%s\"): %s.\n",
@@ -337,11 +373,13 @@ cm_notify_start(struct cm_store_entry *entry, enum cm_notify_event event)
 {
 	struct cm_notify_state *state;
 	struct cm_notify_details details;
+
 	state = talloc_ptrtype(entry, state);
 	if (state != NULL) {
+		memset(state, 0, sizeof(*state));
 		memset(&details, 0, sizeof(details));
 		details.event = event;
-		state->subproc = cm_subproc_start(cm_notify_main,
+		state->subproc = cm_subproc_start(cm_notify_main, state,
 						  NULL, entry, &details);
 		if (state->subproc == NULL) {
 			talloc_free(state);
@@ -353,24 +391,24 @@ cm_notify_start(struct cm_store_entry *entry, enum cm_notify_event event)
 
 /* Get a selectable-for-read descriptor we can poll for status changes. */
 int
-cm_notify_get_fd(struct cm_store_entry *entry, struct cm_notify_state *state)
+cm_notify_get_fd(struct cm_notify_state *state)
 {
-	return cm_subproc_get_fd(entry, state->subproc);
+	return cm_subproc_get_fd(state->subproc);
 }
 
 /* Check if our child process has exited. */
 int
-cm_notify_ready(struct cm_store_entry *entry, struct cm_notify_state *state)
+cm_notify_ready(struct cm_notify_state *state)
 {
-	return cm_subproc_ready(entry, state->subproc);
+	return cm_subproc_ready(state->subproc);
 }
 
 /* Clean up after notification. */
 void
-cm_notify_done(struct cm_store_entry *entry, struct cm_notify_state *state)
+cm_notify_done(struct cm_notify_state *state)
 {
 	if (state->subproc != NULL) {
-		cm_subproc_done(entry, state->subproc);
+		cm_subproc_done(state->subproc);
 	}
 	talloc_free(state);
 }
