@@ -27,6 +27,7 @@
 #include "submit.h"
 #include "submit-int.h"
 #include "store-int.h"
+#include "subproc.h"
 
 /* Start CSR submission using parameters stored in the entry. */
 struct cm_submit_state *
@@ -83,89 +84,82 @@ cm_submit_start(struct cm_store_ca *ca, struct cm_store_entry *entry)
 
 /* Get a selectable-for-read descriptor we can poll for status changes. */
 int
-cm_submit_get_fd(struct cm_store_entry *entry, struct cm_submit_state *state)
+cm_submit_get_fd(struct cm_submit_state *state)
 {
-	struct cm_submit_state_pvt *pvt = (struct cm_submit_state_pvt *) state;
-	return pvt->get_fd(entry, state);
+	return cm_subproc_get_fd(state->subproc);
 }
 
 /* Check if the CSR was submitted to the CA yet, or we figured out that it
  * wasn't possible to accomplish it. */
 int
-cm_submit_ready(struct cm_store_entry *entry, struct cm_submit_state *state)
+cm_submit_ready(struct cm_submit_state *state)
 {
-	struct cm_submit_state_pvt *pvt = (struct cm_submit_state_pvt *) state;
-	return pvt->ready(entry, state);
+	return (*state->ready)(state);
 }
 
 /* Save CA-specific identifier for our submitted request. */
 int
-cm_submit_save_ca_cookie(struct cm_store_entry *entry,
-			 struct cm_submit_state *state)
+cm_submit_save_ca_cookie(struct cm_submit_state *state)
 {
-	struct cm_submit_state_pvt *pvt = (struct cm_submit_state_pvt *) state;
-	return pvt->save_ca_cookie(entry, state);
+	return (*state->save_ca_cookie)(state);
 }
 
 /* Clear CA-specific identifier for our submitted request. */
 int
-cm_submit_clear_ca_cookie(struct cm_store_entry *entry,
-			  struct cm_submit_state *state)
+cm_submit_clear_ca_cookie(struct cm_submit_state *state)
 {
-	talloc_free(entry->cm_ca_cookie);
-	entry->cm_ca_cookie = NULL;
+	talloc_free(state->entry->cm_ca_cookie);
+	state->entry->cm_ca_cookie = NULL;
 	return 0;
 }
 
 /* Check if the certificate was issued. */
 int
-cm_submit_issued(struct cm_store_entry *entry, struct cm_submit_state *state)
+cm_submit_issued(struct cm_submit_state *state)
 {
-	struct cm_submit_state_pvt *pvt = (struct cm_submit_state_pvt *) state;
-	return pvt->issued(entry, state);
+	return (*state->issued)(state);
 }
 
 /* Check if the certificate was rejected. */
 int
-cm_submit_rejected(struct cm_store_entry *entry, struct cm_submit_state *state)
+cm_submit_rejected(struct cm_submit_state *state)
 {
-	struct cm_submit_state_pvt *pvt = (struct cm_submit_state_pvt *) state;
-	return pvt->rejected(entry, state);
+	return (*state->rejected)(state);
+}
+
+/* Check if we're unconfigured or underconfigured. */
+int
+cm_submit_unconfigured(struct cm_submit_state *state)
+{
+	return (*state->unconfigured)(state);
+}
+
+/* Check if we don't support requesting certificates. */
+int
+cm_submit_unsupported(struct cm_submit_state *state)
+{
+	return (*state->unsupported)(state);
 }
 
 /* Check if the CA was unreachable. */
 int
-cm_submit_unconfigured(struct cm_store_entry *entry,
-		       struct cm_submit_state *state)
+cm_submit_unreachable(struct cm_submit_state *state)
 {
-	struct cm_submit_state_pvt *pvt = (struct cm_submit_state_pvt *) state;
-	return pvt->unconfigured(entry, state);
-}
-
-/* Check if the CA was unreachable. */
-int
-cm_submit_unreachable(struct cm_store_entry *entry,
-		      struct cm_submit_state *state)
-{
-	struct cm_submit_state_pvt *pvt = (struct cm_submit_state_pvt *) state;
-	return pvt->unreachable(entry, state);
+	return (*state->unreachable)(state);
 }
 
 /* Done talking to the CA. */
 void
-cm_submit_done(struct cm_store_entry *entry, struct cm_submit_state *state)
+cm_submit_done(struct cm_submit_state *state)
 {
-	struct cm_submit_state_pvt *pvt = (struct cm_submit_state_pvt *) state;
-	pvt->done(entry, state);
+	(*state->done)(state);
 }
 
 /* How long should we wait before talking to the CA again? */
 int
-cm_submit_specified_delay(struct cm_store_entry *entry,
-			  struct cm_submit_state *state)
+cm_submit_specified_delay(struct cm_submit_state *state)
 {
-	struct cm_submit_state_pvt *pvt = (struct cm_submit_state_pvt *) state;
-	return pvt->delay;
+	return state->delay;
 }
 
 /* Concatenate some strings. */
