@@ -26,12 +26,22 @@
 
 #include <dbus/dbus.h>
 
+#include "log.h"
 #include "tdbusm.h"
 
 #define N_(_text) _text
 
 static char empty_string[] = "";
 static const char *empty_string_array[] = {NULL};
+static struct cm_tdbusm_dict **cm_tdbusm_get_d_array(DBusMessageIter *array,
+						     void *parent);
+static struct cm_tdbusm_dict *cm_tdbusm_get_d_item(DBusMessageIter *item,
+						   void *parent);
+static struct cm_tdbusm_dict *cm_tdbusm_get_d_value(DBusMessageIter *item,
+						    void *parent,
+						    struct cm_tdbusm_dict *dict);
+static int cm_tdbusm_append_d(DBusMessage *msg, DBusMessageIter *args,
+			      const struct cm_tdbusm_dict **d);
 
 static int
 cm_tdbusm_array_length(const char **array)
@@ -327,6 +337,7 @@ cm_tdbusm_get_ap(DBusMessage *msg, void *parent, char ***ap)
 	DBusError err;
 	char **tmp;
 	int i;
+
 	*ap = NULL;
 	dbus_error_init(&err);
 	if (dbus_message_get_args(msg, &err,
@@ -344,6 +355,69 @@ cm_tdbusm_get_ap(DBusMessage *msg, void *parent, char ***ap)
 	}
 }
 
+int
+cm_tdbusm_get_ass(DBusMessage *msg, void *parent, char ***ass)
+{
+	DBusMessageIter args, array, element;
+	const char *p, *q;
+	char **ret, **tmp;
+	int i = 0;
+
+	ret = NULL;
+	if (!dbus_message_iter_init(msg, &args)) {
+		talloc_free(ret);
+		return -1;
+	}
+
+	if (dbus_message_iter_get_arg_type(&args) != DBUS_TYPE_ARRAY) {
+		talloc_free(ret);
+		return -1;
+	}
+	memset(&array, 0, sizeof(array));
+	dbus_message_iter_recurse(&args, &array);
+
+	for (;;) {
+		if (dbus_message_iter_get_arg_type(&array) != DBUS_TYPE_STRUCT) {
+			talloc_free(ret);
+			return -1;
+		}
+		dbus_message_iter_recurse(&array, &element);
+
+		if (dbus_message_iter_get_arg_type(&element) != DBUS_TYPE_STRING) {
+			talloc_free(ret);
+			return -1;
+		}
+		p = NULL;
+		dbus_message_iter_get_basic(&element, &p);
+		if (!dbus_message_iter_has_next(&element) ||
+		    !dbus_message_iter_next(&element) ||
+		    (dbus_message_iter_get_arg_type(&element) != DBUS_TYPE_STRING)) {
+			talloc_free(ret);
+			return -1;
+		}
+		q = NULL;
+		dbus_message_iter_get_basic(&element, &q);
+		tmp = talloc_realloc(parent, ret, char *, i + 3);
+		if (tmp == NULL) {
+			talloc_free(ret);
+			return -1;
+		}
+		ret = tmp;
+		ret[i++] = talloc_strdup(ret, p);
+		ret[i++] = talloc_strdup(ret, q);
+		ret[i] = NULL;
+		if (!dbus_message_iter_has_next(&array)) {
+			break;
+		}
+		if (!dbus_message_iter_next(&array)) {
+			talloc_free(ret);
+			return -1;
+		}
+	}
+	*ass = ret;
+	return 0;
+}
+
 int
 cm_tdbusm_get_as(DBusMessage *msg, void *parent, char ***as)
 {
@@ -393,6 +467,40 @@ cm_tdbusm_get_sss(DBusMessage *msg, void *parent, char **s1, char **s2,
 	}
 }
 
+int
+cm_tdbusm_get_ssv(DBusMessage *msg, void *parent, char **s1, char **s2,
+		  enum cm_tdbusm_dict_value_type *type,
+		  union cm_tdbusm_variant *value)
+{
+	DBusMessageIter iter;
+	struct cm_tdbusm_dict *d;
+
+	*s1 = NULL;
+	*s2 = NULL;
+	if (!dbus_message_iter_init(msg, &iter)) {
+		return -1;
+	}
+
+	if (dbus_message_iter_get_arg_type(&iter) != DBUS_TYPE_STRING) {
+		return -1;
+	}
+	dbus_message_iter_get_basic(&iter, s1);
+
+	if (!dbus_message_iter_has_next(&iter) ||
+	    !dbus_message_iter_next(&iter)) {
+		return -1;
+	}
+	d = cm_tdbusm_get_d_item(&iter, parent);
+	if (d == NULL) {
+		return -1;
+	}
+	*s1 = *s1 ? talloc_strdup(parent, *s1) : NULL;
+	*s2 = talloc_strdup(parent, d->key);
+	*type = d->value_type;
+	*value = d->value;
+	return 0;
+}
+
 int
 cm_tdbusm_get_ssb(DBusMessage *msg, void *parent, char **s1, char **s2,
 		  dbus_bool_t *b)
@@ -565,6 +673,82 @@ cm_tdbusm_get_ssas(DBusMessage *msg, void *parent,
 	}
 }
 
+int
+cm_tdbusm_get_ssass(DBusMessage *msg, void *parent,
+		    char **s1, char **s2, char ***ass)
+{
+	DBusMessageIter args, array, element;
+	const char *p, *q, *r, *s;
+	char **ret, **tmp;
+	int i = 0;
+
+	ret = NULL;
+	if (!dbus_message_iter_init(msg, &args)) {
+		return -1;
+	}
+	if (dbus_message_iter_get_arg_type(&args) != DBUS_TYPE_STRING) {
+		return -1;
+	}
+	dbus_message_iter_get_basic(&args, &p);
+	if (!dbus_message_iter_has_next(&args) ||
+	    !dbus_message_iter_next(&args) ||
+	    (dbus_message_iter_get_arg_type(&args) != DBUS_TYPE_STRING)) {
+		return -1;
+	}
+	dbus_message_iter_get_basic(&args, &q);
+	if (!dbus_message_iter_has_next(&args) ||
+	    !dbus_message_iter_next(&args) ||
+	    (dbus_message_iter_get_arg_type(&args) != DBUS_TYPE_ARRAY)) {
+		return -1;
+	}
+
+	memset(&array, 0, sizeof(array));
+	dbus_message_iter_recurse(&args, &array);
+
+	for (;;) {
+		if (dbus_message_iter_get_arg_type(&array) != DBUS_TYPE_STRUCT) {
+			talloc_free(ret);
+			return -1;
+		}
+		dbus_message_iter_recurse(&array, &element);
+
+		if (dbus_message_iter_get_arg_type(&element) != DBUS_TYPE_STRING) {
+			talloc_free(ret);
+			return -1;
+		}
+		r = NULL;
+		dbus_message_iter_get_basic(&element, &r);
+		if (!dbus_message_iter_has_next(&element) ||
+		    !dbus_message_iter_next(&element) ||
+		    (dbus_message_iter_get_arg_type(&element) != DBUS_TYPE_STRING)) {
+			talloc_free(ret);
+			return -1;
+		}
+		s = NULL;
+		dbus_message_iter_get_basic(&element, &s);
+		tmp = talloc_realloc(parent, ret, char *, i + 3);
+		if (tmp == NULL) {
+			talloc_free(ret);
+			return -1;
+		}
+		ret = tmp;
+		ret[i++] = talloc_strdup(ret, r);
+		ret[i++] = talloc_strdup(ret, s);
+		ret[i] = NULL;
+		if (!dbus_message_iter_has_next(&array)) {
+			break;
+		}
+		if (!dbus_message_iter_next(&array)) {
+			talloc_free(ret);
+			return -1;
+		}
+	}
+	*s1 = talloc_strdup(parent, p);
+	*s2 = talloc_strdup(parent, q);
+	*ass = ret;
+	return 0;
+}
+
 int
 cm_tdbusm_get_sssas(DBusMessage *msg, void *parent,
 		    char **s1, char **s2, char **s3, char ***as)
@@ -784,32 +968,17 @@ cm_tdbusm_get_sasasasnas(DBusMessage *msg, void *parent, char **s,
 }
 
 static struct cm_tdbusm_dict *
-cm_tdbusm_get_d_item(DBusMessageIter *item, void *parent)
+cm_tdbusm_get_d_value(DBusMessageIter *item, void *parent,
+		      struct cm_tdbusm_dict *dict)
 {
-	struct cm_tdbusm_dict *dict;
-	char *s, **as;
+	struct cm_tdbusm_dict **dicts;
+	char *s, **as, **ass;
 	int64_t i64;
 	int32_t i32;
 	int16_t i16;
 	int n_values;
-	DBusMessageIter value, sval;
-	dict = talloc_ptrtype(parent, dict);
-	/* Pull out a string. */
-	switch (dbus_message_iter_get_arg_type(item)) {
-	case DBUS_TYPE_STRING:
-		dbus_message_iter_get_basic(item, &s);
-		dict->key = talloc_strdup(dict, s);
-		break;
-	default:
-		talloc_free(dict);
-		return NULL;
-		break;
-	}
-	if (!dbus_message_iter_has_next(item) ||
-	    !dbus_message_iter_next(item)) {
-		talloc_free(dict);
-		return NULL;
-	}
+	DBusMessageIter value, sval, fields;
+
 	/* Pull out a variant. */
 	switch (dbus_message_iter_get_arg_type(item)) {
 	case DBUS_TYPE_VARIANT:
@@ -851,26 +1020,73 @@ cm_tdbusm_get_d_item(DBusMessageIter *item, void *parent)
 			break;
 		/* It can be an array of strings. */
 		case DBUS_TYPE_ARRAY:
-			dict->value_type = cm_tdbusm_dict_as;
 			memset(&sval, 0, sizeof(sval));
 			dbus_message_iter_recurse(&value, &sval);
 			as = NULL;
+			ass = NULL;
 			n_values = 0;
 			for (;;) {
-				/* This had better be a string. */
+				/* This had better be a string or a struct
+				 * containing two strings. */
 				switch (dbus_message_iter_get_arg_type(&sval)) {
 				case DBUS_TYPE_STRING:
+					dict->value_type = cm_tdbusm_dict_as;
 					dbus_message_iter_get_basic(&sval, &s);
 					as = talloc_realloc(dict, as, char *,
 							    n_values + 2);
-					if (as != NULL) {
-						as[n_values] = talloc_strdup(as,
-									     s);
-						n_values++;
-						as[n_values] = NULL;
+					if (as == NULL) {
+						talloc_free(dict);
+						return NULL;
 					}
+					as[n_values] = talloc_strdup(as, s);
+					if (as[n_values] == NULL) {
+						talloc_free(dict);
+						return NULL;
+					}
+					n_values++;
+					as[n_values] = NULL;
+					dict->value.as = as;
+					break;
+				case DBUS_TYPE_STRUCT:
+					dict->value_type = cm_tdbusm_dict_ass;
+					dbus_message_iter_recurse(&sval, &fields);
+					if (dbus_message_iter_get_arg_type(&fields) != DBUS_TYPE_STRING) {
+						talloc_free(dict);
+						return NULL;
+					}
+					ass = talloc_realloc(dict, ass, char *,
+							     n_values + 3);
+					if (ass == NULL) {
+						talloc_free(dict);
+						return NULL;
+					}
+					dbus_message_iter_get_basic(&fields, &s);
+					ass[n_values] = talloc_strdup(ass, s);
+					if (!dbus_message_iter_has_next(&fields) ||
+					    (dbus_message_iter_get_arg_type(&fields) != DBUS_TYPE_STRING) ||
+					    !dbus_message_iter_next(&fields)) {
+						talloc_free(dict);
+						return NULL;
+					}
+					dbus_message_iter_get_basic(&fields, &s);
+					ass[n_values + 1] = talloc_strdup(ass, s);
+					n_values += 2;
+					ass[n_values] = NULL;
+					dict->value.ass = ass;
+					break;
+				case DBUS_TYPE_DICT_ENTRY:
+					dict->value_type = cm_tdbusm_dict_d;
+					dicts = cm_tdbusm_get_d_array(&sval, dict);
+					dict->value.d = (const struct cm_tdbusm_dict **) dicts;
+					break;
+				case DBUS_TYPE_INVALID:
+					dict->value_type = cm_tdbusm_dict_invalid;
+					memset(&dict->value, 0, sizeof(dict->value));
 					break;
 				default:
+					cm_log(6, "Unexpected array member type %c (%d)\n",
+					       dbus_message_iter_get_arg_type(&sval),
+					       dbus_message_iter_get_arg_type(&sval));
 					talloc_free(dict);
 					return NULL;
 					break;
@@ -886,7 +1102,6 @@ cm_tdbusm_get_d_item(DBusMessageIter *item, void *parent)
 					break;
 				}
 			}
-			dict->value.as = as;
 			break;
 		default:
 			/* It had better not be something else. */
@@ -902,12 +1117,46 @@ cm_tdbusm_get_d_item(DBusMessageIter *item, void *parent)
 	}
 	return dict;
 }
+
+static struct cm_tdbusm_dict *
+cm_tdbusm_get_d_item(DBusMessageIter *item, void *parent)
+{
+	struct cm_tdbusm_dict *dict;
+	char *s;
+
+	dict = talloc_ptrtype(parent, dict);
+	if (dict == NULL) {
+		return NULL;
+	}
+	memset(dict, 0, sizeof(*dict));
+
+	/* Pull out a string. */
+	switch (dbus_message_iter_get_arg_type(item)) {
+	case DBUS_TYPE_STRING:
+		dbus_message_iter_get_basic(item, &s);
+		dict->key = talloc_strdup(dict, s);
+		break;
+	default:
+		talloc_free(dict);
+		return NULL;
+		break;
+	}
+	if (!dbus_message_iter_has_next(item) ||
+	    !dbus_message_iter_next(item)) {
+		talloc_free(dict);
+		return NULL;
+	}
+	/* Pull out the corresponding value, whatever it is. */
+	return cm_tdbusm_get_d_value(item, parent, dict);
+}
+
 static struct cm_tdbusm_dict **
 cm_tdbusm_get_d_array(DBusMessageIter *array, void *parent)
 {
 	struct cm_tdbusm_dict *ditem, **dict, **tmp;
 	int n_items;
 	DBusMessageIter item;
+
 	dict = NULL;
 	n_items = 0;
 	for (;;) {
@@ -918,16 +1167,18 @@ cm_tdbusm_get_d_array(DBusMessageIter *array, void *parent)
 			memset(&item, 0, sizeof(item));
 			dbus_message_iter_recurse(array, &item);
 			ditem = cm_tdbusm_get_d_item(&item, parent);
-			if (ditem != NULL) {
-				tmp = talloc_realloc(parent, dict,
-						     struct cm_tdbusm_dict *,
-						     n_items + 2);
-				if (tmp != NULL) {
-					tmp[n_items] = ditem;
-					n_items++;
-					tmp[n_items] = NULL;
-					dict = tmp;
-				}
+			if (ditem == NULL) {
+				talloc_free(dict);
+				return NULL;
+			}
+			tmp = talloc_realloc(parent, dict,
+					     struct cm_tdbusm_dict *,
+					     n_items + 2);
+			if (tmp != NULL) {
+				tmp[n_items] = ditem;
+				n_items++;
+				tmp[n_items] = NULL;
+				dict = tmp;
 			}
 			break;
 		default:
@@ -954,6 +1205,7 @@ cm_tdbusm_get_d(DBusMessage *msg, void *parent, struct cm_tdbusm_dict ***d)
 	struct cm_tdbusm_dict **tdicts, **dicts, **tmp;
 	DBusMessageIter args, array;
 	int i, n_dicts;
+
 	*d = NULL;
 	dicts = NULL;
 	n_dicts = 0;
@@ -965,25 +1217,27 @@ cm_tdbusm_get_d(DBusMessage *msg, void *parent, struct cm_tdbusm_dict ***d)
 				memset(&array, 0, sizeof(array));
 				dbus_message_iter_recurse(&args, &array);
 				tdicts = cm_tdbusm_get_d_array(&array, parent);
-				if (tdicts != NULL) {
-					for (i = 0; tdicts[i] != NULL; i++) {
-						continue;
-					}
-					tmp = talloc_realloc(parent, dicts,
-							     struct cm_tdbusm_dict *,
-							     n_dicts + i + 1);
-					if (tmp != NULL) {
-						memcpy(tmp + n_dicts,
-						       tdicts,
-						       i * sizeof(tdicts[0]));
-						n_dicts += i;
-						tmp[n_dicts] = NULL;
-						dicts = tmp;
-					} else {
-						talloc_free(tdicts);
-						talloc_free(dicts);
-						return -1;
-					}
+				if (tdicts == NULL) {
+					talloc_free(dicts);
+					return -1;
+				}
+				for (i = 0; tdicts[i] != NULL; i++) {
+					continue;
+				}
+				tmp = talloc_realloc(parent, dicts,
+						     struct cm_tdbusm_dict *,
+						     n_dicts + i + 1);
+				if (tmp != NULL) {
+					memcpy(tmp + n_dicts,
+					       tdicts,
+					       i * sizeof(tdicts[0]));
+					n_dicts += i;
+					tmp[n_dicts] = NULL;
+					dicts = tmp;
+				} else {
+					talloc_free(tdicts);
+					talloc_free(dicts);
+					return -1;
 				}
 				break;
 			default:
@@ -1273,6 +1527,39 @@ cm_tdbusm_set_as(DBusMessage *msg, const char **as)
 	}
 }
 
+int
+cm_tdbusm_set_ass(DBusMessage *msg, const char **ss)
+{
+	DBusMessageIter args, array, entry;
+	const char *p;
+	int i;
+
+	memset(&args, 0, sizeof(args));
+	dbus_message_iter_init_append(msg, &args);
+	memset(&array, 0, sizeof(array));
+	dbus_message_iter_open_container(&args,
+					 DBUS_TYPE_ARRAY,
+					 DBUS_STRUCT_BEGIN_CHAR_AS_STRING
+					 DBUS_TYPE_STRING_AS_STRING
+					 DBUS_TYPE_STRING_AS_STRING
+					 DBUS_STRUCT_END_CHAR_AS_STRING,
+					 &array);
+	for (i = 0;
+	     (ss != NULL) && (ss[i] != NULL) && (ss[i + 1] != NULL);
+	     i += 2) {
+		memset(&entry, 0, sizeof(entry));
+		dbus_message_iter_open_container(&array, DBUS_TYPE_STRUCT,
+						 NULL, &entry);
+		p = ss[i];
+		dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &p);
+		p = ss[i + 1];
+		dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &p);
+		dbus_message_iter_close_container(&array, &entry);
+	}
+	dbus_message_iter_close_container(&args, &array);
+	return (i > 0) ? 0 : -1;
+}
+
 int
 cm_tdbusm_set_sss(DBusMessage *msg, const char *s1, const char *s2,
 		  const char *s3)
@@ -1350,6 +1637,56 @@ cm_tdbusm_set_ssas(DBusMessage *msg,
 	}
 }
 
+int
+cm_tdbusm_set_ssass(DBusMessage *msg,
+		    const char *s1, const char *s2, const char **ass)
+{
+	DBusMessageIter args, elt, fields;
+	int i;
+
+	memset(&args, 0, sizeof(args));
+	if (s1 == NULL) {
+		s1 = empty_string;
+	}
+	if (s2 == NULL) {
+		s2 = empty_string;
+	}
+	if (ass == NULL) {
+		ass = empty_string_array;
+	}
+	if (dbus_message_append_args(msg,
+				     DBUS_TYPE_STRING, &s1,
+				     DBUS_TYPE_STRING, &s2,
+				     DBUS_TYPE_INVALID)) {
+		dbus_message_iter_init_append(msg, &args);
+		dbus_message_iter_open_container(&args,
+						 DBUS_TYPE_ARRAY,
+						 DBUS_STRUCT_BEGIN_CHAR_AS_STRING
+						 DBUS_TYPE_STRING_AS_STRING
+						 DBUS_TYPE_STRING_AS_STRING
+						 DBUS_STRUCT_END_CHAR_AS_STRING,
+						 &elt);
+		for (i = 0;
+		     (ass != NULL) && (ass[i] != NULL) && (ass[i + 1] != NULL);
+		     i += 2) {
+			dbus_message_iter_open_container(&elt,
+							 DBUS_TYPE_STRUCT,
+							 NULL,
+							 &fields);
+			dbus_message_iter_append_basic(&fields,
+						       DBUS_TYPE_STRING,
+						       &ass[i]);
+			dbus_message_iter_append_basic(&fields,
+						       DBUS_TYPE_STRING,
+						       &ass[i + 1]);
+			dbus_message_iter_close_container(&elt, &fields);
+		}
+		dbus_message_iter_close_container(&args, &elt);
+		return 0;
+	} else {
+		return -1;
+	}
+}
 
 int
 cm_tdbusm_set_ssoas(DBusMessage *msg,
@@ -1495,19 +1832,168 @@ cm_tdbusm_set_sasasasnas(DBusMessage *msg, const char *s,
 }
 
 static int
-cm_tdbusm_set_osd(DBusMessage *msg,
-		  const char *s, const struct cm_tdbusm_dict **d)
+cm_tdbusm_append_d_value(DBusMessage *msg, DBusMessageIter *args,
+			 enum cm_tdbusm_dict_value_type value_type,
+			 const union cm_tdbusm_variant *value)
 {
-	DBusMessageIter args, array, entry, val, elt;
-	int i;
+	DBusMessageIter val, elt, fields;
+	int subs = 0;
 	int64_t l;
-	memset(&args, 0, sizeof(args));
-	dbus_message_iter_init_append(msg, &args);
-	if (s != NULL) {
-		dbus_message_iter_append_basic(&args, DBUS_TYPE_STRING, &s);
+
+	memset(&val, 0, sizeof(val));
+	switch (value_type) {
+	case cm_tdbusm_dict_invalid:
+		dbus_message_iter_open_container(args,
+						 DBUS_TYPE_VARIANT,
+						 DBUS_TYPE_INVALID_AS_STRING,
+						 &val);
+		dbus_message_iter_append_basic(&val,
+					       DBUS_TYPE_INVALID,
+					       NULL);
+		dbus_message_iter_close_container(args, &val);
+		break;
+	case cm_tdbusm_dict_b:
+		dbus_message_iter_open_container(args,
+						 DBUS_TYPE_VARIANT,
+						 DBUS_TYPE_BOOLEAN_AS_STRING,
+						 &val);
+		dbus_message_iter_append_basic(&val,
+					       DBUS_TYPE_BOOLEAN,
+					       &value->b);
+		dbus_message_iter_close_container(args, &val);
+		break;
+	case cm_tdbusm_dict_n:
+		dbus_message_iter_open_container(args,
+						 DBUS_TYPE_VARIANT,
+						 DBUS_TYPE_INT64_AS_STRING,
+						 &val);
+		l = value->n;
+		dbus_message_iter_append_basic(&val,
+					       DBUS_TYPE_INT64,
+					       &l);
+		dbus_message_iter_close_container(args, &val);
+		break;
+	case cm_tdbusm_dict_p:
+		dbus_message_iter_open_container(args,
+						 DBUS_TYPE_VARIANT,
+						 DBUS_TYPE_OBJECT_PATH_AS_STRING,
+						 &val);
+		dbus_message_iter_append_basic(&val,
+					       DBUS_TYPE_OBJECT_PATH,
+					       &value->s);
+		dbus_message_iter_close_container(args, &val);
+		break;
+	case cm_tdbusm_dict_s:
+		dbus_message_iter_open_container(args,
+						 DBUS_TYPE_VARIANT,
+						 DBUS_TYPE_STRING_AS_STRING,
+						 &val);
+		dbus_message_iter_append_basic(&val,
+					       DBUS_TYPE_STRING,
+					       &value->s);
+		dbus_message_iter_close_container(args, &val);
+		break;
+	case cm_tdbusm_dict_as:
+		dbus_message_iter_open_container(args,
+						 DBUS_TYPE_VARIANT,
+						 DBUS_TYPE_ARRAY_AS_STRING
+						 DBUS_TYPE_STRING_AS_STRING,
+						 &val);
+		memset(&elt, 0, sizeof(elt));
+		dbus_message_iter_open_container(&val,
+						 DBUS_TYPE_ARRAY,
+						 DBUS_TYPE_STRING_AS_STRING,
+						 &elt);
+		for (l = 0;
+		     (value->as != NULL) && (value->as[l] != NULL);
+		     l++) {
+			dbus_message_iter_append_basic(&elt,
+						       DBUS_TYPE_STRING,
+						       &value->as[l]);
+		}
+		dbus_message_iter_close_container(&val, &elt);
+		dbus_message_iter_close_container(args, &val);
+		break;
+	case cm_tdbusm_dict_ass:
+		dbus_message_iter_open_container(args,
+						 DBUS_TYPE_VARIANT,
+						 DBUS_TYPE_ARRAY_AS_STRING
+						 DBUS_STRUCT_BEGIN_CHAR_AS_STRING
+						 DBUS_TYPE_STRING_AS_STRING
+						 DBUS_TYPE_STRING_AS_STRING
+						 DBUS_STRUCT_END_CHAR_AS_STRING,
+						 &val);
+		memset(&elt, 0, sizeof(elt));
+		dbus_message_iter_open_container(&val,
+						 DBUS_TYPE_ARRAY,
+						 DBUS_STRUCT_BEGIN_CHAR_AS_STRING
+						 DBUS_TYPE_STRING_AS_STRING
+						 DBUS_TYPE_STRING_AS_STRING
+						 DBUS_STRUCT_END_CHAR_AS_STRING,
+						 &elt);
+		for (l = 0;
+		     (value->ass != NULL) &&
+		     (value->ass[l] != NULL) &&
+		     (value->ass[l + 1] != NULL);
+		     l += 2) {
+			memset(&fields, 0, sizeof(fields));
+			dbus_message_iter_open_container(&elt,
+							 DBUS_TYPE_STRUCT,
+							 NULL,
+							 &fields);
+			dbus_message_iter_append_basic(&fields,
+						       DBUS_TYPE_STRING,
+						       &value->ass[l]);
+			dbus_message_iter_append_basic(&fields,
+						       DBUS_TYPE_STRING,
+						       &value->ass[l + 1]);
+			dbus_message_iter_close_container(&elt, &fields);
+		}
+		dbus_message_iter_close_container(&val, &elt);
+		dbus_message_iter_close_container(args, &val);
+		break;
+	case cm_tdbusm_dict_d:
+		dbus_message_iter_open_container(args,
+						 DBUS_TYPE_VARIANT,
+						 DBUS_TYPE_ARRAY_AS_STRING
+						 DBUS_DICT_ENTRY_BEGIN_CHAR_AS_STRING
+						 DBUS_TYPE_STRING_AS_STRING
+						 DBUS_TYPE_VARIANT_AS_STRING
+						 DBUS_DICT_ENTRY_END_CHAR_AS_STRING,
+						 &val);
+		subs += cm_tdbusm_append_d(msg, &val,
+					   (const struct cm_tdbusm_dict **) value->d);
+		dbus_message_iter_close_container(args, &val);
+		break;
 	}
+	return subs;
+}
+
+static int
+cm_tdbusm_append_d_item(DBusMessage *msg, DBusMessageIter *args,
+			const struct cm_tdbusm_dict *d)
+{
+	DBusMessageIter entry;
+	int subs = 0;
+
+	memset(&entry, 0, sizeof(entry));
+	dbus_message_iter_open_container(args, DBUS_TYPE_DICT_ENTRY, NULL,
+					 &entry);
+	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &d->key);
+	subs = cm_tdbusm_append_d_value(msg, &entry, d->value_type, &d->value);
+	dbus_message_iter_close_container(args, &entry);
+	return subs;
+}
+
+static int
+cm_tdbusm_append_d(DBusMessage *msg, DBusMessageIter *args,
+		   const struct cm_tdbusm_dict **d)
+{
+	DBusMessageIter array;
+	int i, subs = 0;
+
 	memset(&array, 0, sizeof(array));
-	dbus_message_iter_open_container(&args,
+	dbus_message_iter_open_container(args,
 					 DBUS_TYPE_ARRAY,
 					 DBUS_DICT_ENTRY_BEGIN_CHAR_AS_STRING
 					 DBUS_TYPE_STRING_AS_STRING
@@ -1515,81 +2001,25 @@ cm_tdbusm_set_osd(DBusMessage *msg,
 					 DBUS_DICT_ENTRY_END_CHAR_AS_STRING,
 					 &array);
 	for (i = 0; (d != NULL) && (d[i] != NULL); i++) {
-		memset(&entry, 0, sizeof(entry));
-		dbus_message_iter_open_container(&array, DBUS_TYPE_DICT_ENTRY,
-						 NULL,
-						 &entry);
-		dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING,
-					       &d[i]->key);
-		memset(&val, 0, sizeof(val));
-		switch (d[i]->value_type) {
-		case cm_tdbusm_dict_b:
-			dbus_message_iter_open_container(&entry,
-							 DBUS_TYPE_VARIANT,
-							 DBUS_TYPE_BOOLEAN_AS_STRING,
-							 &val);
-			dbus_message_iter_append_basic(&val,
-						       DBUS_TYPE_BOOLEAN,
-						       &d[i]->value.b);
-			dbus_message_iter_close_container(&entry, &val);
-			break;
-		case cm_tdbusm_dict_n:
-			dbus_message_iter_open_container(&entry,
-							 DBUS_TYPE_VARIANT,
-							 DBUS_TYPE_INT64_AS_STRING,
-							 &val);
-			l = d[i]->value.n;
-			dbus_message_iter_append_basic(&val,
-						       DBUS_TYPE_INT64,
-						       &l);
-			dbus_message_iter_close_container(&entry, &val);
-			break;
-		case cm_tdbusm_dict_p:
-			dbus_message_iter_open_container(&entry,
-							 DBUS_TYPE_VARIANT,
-							 DBUS_TYPE_OBJECT_PATH_AS_STRING,
-							 &val);
-			dbus_message_iter_append_basic(&val,
-						       DBUS_TYPE_OBJECT_PATH,
-						       &d[i]->value.s);
-			dbus_message_iter_close_container(&entry, &val);
-			break;
-		case cm_tdbusm_dict_s:
-			dbus_message_iter_open_container(&entry,
-							 DBUS_TYPE_VARIANT,
-							 DBUS_TYPE_STRING_AS_STRING,
-							 &val);
-			dbus_message_iter_append_basic(&val,
-						       DBUS_TYPE_STRING,
-						       &d[i]->value.s);
-			dbus_message_iter_close_container(&entry, &val);
-			break;
-		case cm_tdbusm_dict_as:
-			dbus_message_iter_open_container(&entry,
-							 DBUS_TYPE_VARIANT,
-							 DBUS_TYPE_ARRAY_AS_STRING
-							 DBUS_TYPE_STRING_AS_STRING,
-							 &val);
-			memset(&elt, 0, sizeof(elt));
-			dbus_message_iter_open_container(&val,
-							 DBUS_TYPE_ARRAY,
-							 DBUS_TYPE_STRING_AS_STRING,
-							 &elt);
-			for (l = 0;
-			     (d[i]->value.as != NULL) &&
-			     (d[i]->value.as[l] != NULL);
-			     l++) {
-				dbus_message_iter_append_basic(&elt,
-							       DBUS_TYPE_STRING,
-							       &d[i]->value.as[l]);
-			}
-			dbus_message_iter_close_container(&val, &elt);
-			dbus_message_iter_close_container(&entry, &val);
-			break;
-		}
-		dbus_message_iter_close_container(&array, &entry);
+		subs += cm_tdbusm_append_d_item(msg, &array, d[i]);
 	}
-	dbus_message_iter_close_container(&args, &array);
+	dbus_message_iter_close_container(args, &array);
+	return i + subs;
+}
+
+static int
+cm_tdbusm_set_osd(DBusMessage *msg,
+		  const char *s, const struct cm_tdbusm_dict **d)
+{
+	DBusMessageIter args;
+	int i;
+
+	memset(&args, 0, sizeof(args));
+	dbus_message_iter_init_append(msg, &args);
+	if (s != NULL) {
+		dbus_message_iter_append_basic(&args, DBUS_TYPE_STRING, &s);
+	}
+	i = cm_tdbusm_append_d(msg, &args, d);
 	return (i > 0) ? 0 : -1;
 }
 
@@ -1599,6 +2029,21 @@ cm_tdbusm_set_d(DBusMessage *msg, const struct cm_tdbusm_dict **d)
 	return cm_tdbusm_set_osd(msg, NULL, d);
 }
 
+int
+cm_tdbusm_set_v(DBusMessage *msg, enum cm_tdbusm_dict_value_type value_type,
+		const union cm_tdbusm_variant *value)
+{
+	DBusMessageIter args;
+	int i = 0;
+
+	memset(&args, 0, sizeof(args));
+	dbus_message_iter_init_append(msg, &args);
+	if (value != NULL) {
+		i = cm_tdbusm_append_d_value(msg, &args, value_type, value);
+	}
+	return (i > 0) ? 0 : -1;
+}
+
 int
 cm_tdbusm_set_sd(DBusMessage *msg,
 		 const char *s, const struct cm_tdbusm_dict **d)
